# Copyright © Michal Čihař <michal@weblate.org>
#
# SPDX-License-Identifier: GPL-3.0-or-later

from django.utils.translation import gettext_lazy as _

from weblate.addons.base import BaseAddon
from weblate.addons.events import EVENT_COMPONENT_UPDATE, EVENT_UNIT_PRE_CREATE
from weblate.addons.forms import BulkEditAddonForm
from weblate.trans.bulk import bulk_perform
from weblate.trans.models import Unit
from weblate.utils.state import STATE_FUZZY, STATE_TRANSLATED


class FlagBase(BaseAddon):
    events = (EVENT_UNIT_PRE_CREATE,)
    icon = "flag.svg"

    @classmethod
    def can_install(cls, component, user):
        # Following formats support fuzzy flag, so avoid messing up with them
        if component.file_format in {"ts", "po", "po-mono"}:
            return False
        return super().can_install(component, user)


class SourceEditAddon(FlagBase):
    name = "weblate.flags.source_edit"
    verbose = _('Flag new source strings as "Needs editing"')
    description = _(
        "Whenever a new source string is imported from the VCS, it is "
        "flagged as needing editing in Weblate. This way you can easily "
        "filter and edit source strings written by the developers."
    )
    compat = {
        "edit_template": {True},
    }

    def unit_pre_create(self, unit):
        if (
            unit.translation.is_template
            and unit.state >= STATE_TRANSLATED
            and not unit.readonly
        ):
            unit.state = STATE_FUZZY


class TargetEditAddon(FlagBase):
    name = "weblate.flags.target_edit"
    verbose = _('Flag new translations as "Needs editing"')
    description = _(
        "Whenever a new translatable string is imported from the VCS, it is "
        "flagged as needing editing in Weblate. This way you can easily "
        "filter and edit translations created by the developers."
    )

    def unit_pre_create(self, unit):
        if (
            not unit.translation.is_template
            and unit.state >= STATE_TRANSLATED
            and not unit.readonly
        ):
            unit.state = STATE_FUZZY


class SameEditAddon(FlagBase):
    name = "weblate.flags.same_edit"
    verbose = _('Flag unchanged translations as "Needs editing"')
    description = _(
        "Whenever a new translatable string is imported from the VCS and it matches a "
        "source string, it is flagged as needing editing in Weblate. Especially "
        "useful for file formats that include source strings for untranslated strings."
    )

    def unit_pre_create(self, unit):
        if (
            not unit.translation.is_template
            and unit.source == unit.target
            and "ignore-same" not in unit.all_flags
            and unit.state >= STATE_TRANSLATED
            and not unit.readonly
        ):
            unit.state = STATE_FUZZY


class BulkEditAddon(BaseAddon):
    events = (EVENT_COMPONENT_UPDATE,)
    name = "weblate.flags.bulk"
    verbose = _("Bulk edit")
    description = _("Bulk edit flags, labels, or states of strings.")
    settings_form = BulkEditAddonForm
    multiple = True

    def component_update(self, component):
        label_set = component.project.label_set
        bulk_perform(
            None,
            Unit.objects.filter(translation__component=component),
            components=[component],
            query=self.instance.configuration["q"],
            target_state=self.instance.configuration["state"],
            add_flags=self.instance.configuration["add_flags"],
            remove_flags=self.instance.configuration["remove_flags"],
            add_labels=label_set.filter(
                name__in=self.instance.configuration["add_labels"]
            ),
            remove_labels=label_set.filter(
                name__in=self.instance.configuration["remove_labels"]
            ),
            project=component.project,
        )



# Copyright © Michal Čihař <michal@weblate.org>
#
# SPDX-License-Identifier: GPL-3.0-or-later

from django import forms
from django.utils.translation import gettext_lazy as _


class TOSForm(forms.Form):
    confirm = forms.BooleanField(
        label=_("I agree with the Terms of Service document"), required=True
    )
    next = forms.CharField(required=False, widget=forms.HiddenInput)



# Copyright © Michal Čihař <michal@weblate.org>
#
# SPDX-License-Identifier: GPL-3.0-or-later

"""Charting library for Weblate."""

from typing import Optional

from django.http import JsonResponse

from weblate.metrics.models import Metric
from weblate.metrics.wrapper import MetricsWrapper


def monthly_activity_json(
    request,
    project: Optional[str] = None,
    component: Optional[str] = None,
    lang: Optional[str] = None,
    user: Optional[str] = None,
):
    """Return monthly activity for matching changes as json."""
    metrics = MetricsWrapper(None, Metric.SCOPE_GLOBAL, 0)
    return JsonResponse(data=metrics.daily_activity, safe=False)



# Copyright © Michal Čihař <michal@weblate.org>
#
# SPDX-License-Identifier: GPL-3.0-or-later

# Generated by Django 3.2 on 2022-01-12 08:54

import django.db.models.deletion
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ("trans", "0143_alter_unit_labels"),
    ]

    operations = [
        migrations.AddField(
            model_name="projecttoken",
            name="user",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                to=settings.AUTH_USER_MODEL,
            ),
        ),
    ]



# Copyright © Michal Čihař <michal@weblate.org>
#
# SPDX-License-Identifier: GPL-3.0-or-later

import subprocess
from functools import lru_cache

from django.core.management.utils import find_command

GIT_PATHS = [
    "/usr/lib/git",
    "/usr/lib/git-core",
    "/usr/libexec/git",
    "/usr/libexec/git-core",
]


@lru_cache(maxsize=None)
def find_git_http_backend():
    """Find Git HTTP back-end."""
    try:
        path = subprocess.run(
            ["git", "--exec-path"],
            text=True,
            check=True,
            capture_output=True,
        ).stdout.strip()
        if path:
            GIT_PATHS.insert(0, path)
    except OSError:
        pass

    return find_command("git-http-backend", path=GIT_PATHS)



# Copyright © Michal Čihař <michal@weblate.org>
#
# SPDX-License-Identifier: GPL-3.0-or-later

"""Test for AJAX/JS views."""

from django.urls import reverse

from weblate.trans.tests.test_views import FixtureTestCase


class JSViewsTest(FixtureTestCase):
    """Testing of AJAX/JS views."""

    def test_get_unit_translations(self):
        unit = self.get_unit()
        response = self.client.get(
            reverse("js-unit-translations", kwargs={"unit_id": unit.id})
        )
        self.assertContains(response, 'href="/translate/')



# Copyright © Michal Čihař <michal@weblate.org>
#
# SPDX-License-Identifier: GPL-3.0-or-later

# Generated by Django 4.1a1 on 2022-05-19 11:27

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("trans", "0150_convert_context"),
    ]

    operations = [
        migrations.AddField(
            model_name="project",
            name="machinery_settings",
            field=models.JSONField(default=dict, blank=True),
        ),
    ]



# Copyright © Michal Čihař <michal@weblate.org>
#
# SPDX-License-Identifier: GPL-3.0-or-later

"""Test for contributor agreement management."""

from weblate.trans.models import ContributorAgreement
from weblate.trans.tests.test_views import FixtureTestCase


class AgreementTest(FixtureTestCase):
    def test_basic(self):
        self.assertFalse(
            ContributorAgreement.objects.has_agreed(self.user, self.component)
        )
        ContributorAgreement.objects.create(self.user, self.component)
        self.assertTrue(
            ContributorAgreement.objects.has_agreed(self.user, self.component)
        )

    def test_perms(self):
        self.assertTrue(self.user.has_perm("unit.edit", self.component))
        self.component.agreement = "CLA"
        self.user.clear_cache()
        self.assertFalse(self.user.has_perm("unit.edit", self.component))
        ContributorAgreement.objects.create(self.user, self.component)
        self.user.clear_cache()
        self.assertTrue(self.user.has_perm("unit.edit", self.component))



# Copyright © Michal Čihař <michal@weblate.org>
#
# SPDX-License-Identifier: GPL-3.0-or-later

from weblate.trans.management.commands import WeblateComponentCommand


class Command(WeblateComponentCommand):
    help = "pushes all changes to upstream repository"
    needs_repo = True

    def add_arguments(self, parser):
        super().add_arguments(parser)
        parser.add_argument(
            "--force-commit",
            action="store_true",
            dest="force_commit",
            default=False,
            help="Forces committing pending changes",
        )

    def handle(self, *args, **options):
        for component in self.get_components(**options):
            component.do_push(None, force_commit=options["force_commit"])



# Copyright © Michal Čihař <michal@weblate.org>
#
# SPDX-License-Identifier: GPL-3.0-or-later

# Generated by Django 4.0.4 on 2022-05-17 08:11

from django.db import migrations

from weblate.utils.hash import calculate_hash


def fix_convert_context(apps, schema_editor):
    Translation = apps.get_model("trans", "Translation")
    Unit = apps.get_model("trans", "Unit")
    translations = Translation.objects.using(schema_editor.connection.alias).filter(
        component__file_format__in=(
            "html",
            "idml",
            "rc",
            "txt",
            "dokuwiki",
            "mediawiki",
        )
    )
    for translation in translations:
        hashes = set()
        duplicate_hashes = set()
        units = translation.unit_set.all()
        for unit in units:
            unit.tmp_id_hash = calculate_hash(unit.source, "")
            unit.location = unit.context
            if unit.tmp_id_hash in hashes:
                duplicate_hashes.add(unit.tmp_id_hash)
            hashes.add(unit.tmp_id_hash)

        for unit in units:
            if unit.tmp_id_hash in duplicate_hashes:
                unit.id_hash = calculate_hash(unit.source, unit.context)
            else:
                unit.id_hash = unit.tmp_id_hash
                unit.context = ""

        Unit.objects.using(schema_editor.connection.alias).bulk_update(
            units, ["location", "context", "id_hash"]
        )


class Migration(migrations.Migration):

    dependencies = [
        ("trans", "0149_component_pull_message"),
    ]

    operations = [migrations.RunPython(fix_convert_context, elidable=True)]



# Copyright © Michal Čihař <michal@weblate.org>
#
# SPDX-License-Identifier: GPL-3.0-or-later

import re

from pyparsing import Optional, QuotedString, Regex, ZeroOrMore


def single_value_flag(func):
    def parse_values(val):
        if not val:
            raise ValueError("Missing required parameter")
        if len(val) > 1:
            raise ValueError("Too many parameters")
        return func(val[0])

    return parse_values


def multi_value_flag(func, minimum=1, maximum=None, modulo=None):
    def parse_values(val):
        if modulo and len(val) % modulo != 0:
            raise ValueError("Number of parameter is not even")
        if minimum and len(val) < minimum:
            raise ValueError("Missing required parameter")
        if maximum and len(val) > maximum:
            raise ValueError("Too many parameters")
        return [func(x) for x in val]

    return parse_values


class RawQuotedString(QuotedString):
    def __init__(self, quote_char, esc_char="\\"):
        super().__init__(quote_char, esc_char=esc_char, convert_whitespace_escapes=True)
        # unlike the QuotedString this replaces only escaped quotes and not all chars
        self.escCharReplacePattern = (
            re.escape(esc_char)
            + "(["
            + re.escape(quote_char)
            + re.escape(esc_char)
            + "])"
        )


SYNTAXCHARS = {",", ":", '"', "'", "\\"}

FlagName = Regex(r"""[^,:"'\\ \r\n\t]([^,:"'\\]*[^,:"'\\ \r\n\t])?""")

RegexString = "r" + RawQuotedString('"')

FlagParam = Optional(
    RegexString | FlagName | RawQuotedString("'") | RawQuotedString('"')
)

Flag = FlagName + ZeroOrMore(":" + FlagParam)

FlagsParser = Optional(Flag) + ZeroOrMore("," + Optional(Flag))



# Copyright © Michal Čihař <michal@weblate.org>
#
# SPDX-License-Identifier: GPL-3.0-or-later

"""Font handling wrapper."""


import os
from functools import lru_cache
from io import BytesIO
from tempfile import NamedTemporaryFile

import cairo
import gi
from django.conf import settings
from django.core.cache import cache
from django.utils.html import format_html
from PIL import ImageFont

from weblate.utils.checks import weblate_check
from weblate.utils.data import data_dir

gi.require_version("PangoCairo", "1.0")
gi.require_version("Pango", "1.0")
from gi.repository import Pango, PangoCairo  # noqa:E402,I001 isort:skip

FONTCONFIG_CONFIG = """<?xml version="1.0"?>
<!DOCTYPE fontconfig SYSTEM "fonts.dtd">
<fontconfig>
    <cachedir>{}</cachedir>
    <dir>{}</dir>
    <dir>{}</dir>
    <dir>{}</dir>
    <dir>{}</dir>
    <dir>{}</dir>
    <config>
        <rescan>
            <int>30</int>
        </rescan>
    </config>

    <alias>
        <family>sans-serif</family>
        <prefer>
            <family>Source Sans 3</family>
            <family>DejaVu Sans</family>
            <family>Noto Sans</family>
            <family>Droid Sans Fallback</family>
        </prefer>
    </alias>

    <alias>
        <family>Source Sans 3</family>
        <default><family>sans-serif</family></default>
    </alias>

    <alias>
        <family>DejaVu Sans</family>
        <default><family>sans-serif</family></default>
    </alias>

    <!--
     Synthetic emboldening for fonts that do not have bold face available
    -->
    <match target="font">
        <test name="weight" compare="less_eq">
            <const>medium</const>
        </test>
        <test target="pattern" name="weight" compare="more_eq">
            <const>bold</const>
        </test>
        <edit name="embolden" mode="assign">
            <bool>true</bool>
        </edit>
        <edit name="weight" mode="assign">
            <const>bold</const>
        </edit>
    </match>
    <!--
      Enable slight hinting for better sub-pixel rendering
    -->
    <match target="pattern">
      <edit name="hintstyle" mode="append"><const>hintslight</const></edit>
    </match>
</fontconfig>
"""

FONT_WEIGHTS = {
    "normal": Pango.Weight.NORMAL,
    "light": Pango.Weight.LIGHT,
    "bold": Pango.Weight.BOLD,
    "": None,
}


def configure_fontconfig():
    """Configures fontconfig to use custom configuration."""
    if getattr(configure_fontconfig, "is_configured", False):
        return

    fonts_dir = data_dir("fonts")
    config_name = os.path.join(fonts_dir, "fonts.conf")

    if not os.path.exists(fonts_dir):
        os.makedirs(fonts_dir)

    # Generate the configuration
    with open(config_name, "w") as handle:
        handle.write(
            FONTCONFIG_CONFIG.format(
                data_dir("cache", "fonts"),
                fonts_dir,
                os.path.join(settings.STATIC_ROOT, "vendor", "font-source", "TTF"),
                os.path.join(settings.STATIC_ROOT, "vendor", "font-dejavu"),
                os.path.join(settings.STATIC_ROOT, "font-noto"),
                os.path.join(settings.STATIC_ROOT, "font-droid"),
            )
        )

    # Inject into environment
    os.environ["FONTCONFIG_FILE"] = config_name

    configure_fontconfig.is_configured = True


def get_font_weight(weight):
    return FONT_WEIGHTS[weight]


@lru_cache(maxsize=512)
def render_size(font, weight, size, spacing, text, width=1000, lines=1, cache_key=None):
    """Check whether rendered text fits."""
    configure_fontconfig()

    # Setup Pango/Cairo
    surface = cairo.ImageSurface(cairo.FORMAT_ARGB32, width * 2, lines * size * 4)
    context = cairo.Context(surface)
    layout = PangoCairo.create_layout(context)

    # Load and configure font
    fontdesc = Pango.FontDescription.from_string(font)
    fontdesc.set_absolute_size(size * Pango.SCALE)
    if weight:
        fontdesc.set_weight(weight)
    layout.set_font_description(fontdesc)

    # This seems to be only way to set letter spacing
    # See https://stackoverflow.com/q/55533312/225718
    layout.set_markup(
        format_html(
            '<span letter_spacing="{}">{}</span>',
            spacing,
            text,
        )
    )

    # Set width and line wrapping
    layout.set_width(width * Pango.SCALE)
    layout.set_wrap(Pango.WrapMode.WORD)

    # Calculate dimensions
    line_count = layout.get_line_count()
    pixel_size = layout.get_pixel_size()

    # Show text
    PangoCairo.show_layout(context, layout)

    # Render box around desired size
    expected_height = lines * pixel_size.height / line_count
    context.new_path()
    context.set_source_rgb(0.8, 0.8, 0.8)
    context.set_line_width(1)
    context.move_to(1, 1)
    context.line_to(width, 1)
    context.line_to(width, expected_height)
    context.line_to(1, expected_height)
    context.line_to(1, 1)
    context.stroke()

    # Render box about actual size
    context.new_path()
    if pixel_size.width > width or line_count > lines:
        context.set_source_rgb(246 / 255, 102 / 255, 76 / 255)
    else:
        context.set_source_rgb(0.4, 0.4, 0.4)
    context.set_line_width(1)
    context.move_to(1, 1)
    context.line_to(pixel_size.width, 1)
    context.line_to(pixel_size.width, pixel_size.height)
    context.line_to(1, pixel_size.height)
    context.line_to(1, 1)
    context.stroke()

    if cache_key:
        with BytesIO() as buff:
            surface.write_to_png(buff)
            cache.set(cache_key, buff.getvalue())

    return pixel_size, line_count


def check_render_size(font, weight, size, spacing, text, width, lines, cache_key=None):
    """Checks whether rendered text fits."""
    size, actual_lines = render_size(
        font, weight, size, spacing, text, width, lines, cache_key
    )
    return size.width <= width and actual_lines <= lines


def get_font_name(filelike):
    """Returns tuple of font family and style, for example ('Ubuntu', 'Regular')."""
    if not hasattr(filelike, "loaded_font"):
        # The tempfile creation is workaroud for Pillow crashing on invalid font
        # see https://github.com/python-pillow/Pillow/issues/3853
        # Once this is fixed, it should be possible to directly operate on filelike
        temp = NamedTemporaryFile(delete=False)
        try:
            temp.write(filelike.read())
            filelike.seek(0)
            temp.close()
            filelike.loaded_font = ImageFont.truetype(temp.name)
        finally:
            os.unlink(temp.name)
    return filelike.loaded_font.getname()


def check_fonts(app_configs=None, **kwargs):
    """Checks font rendering."""
    try:
        render_size("DejaVu Sans", Pango.Weight.NORMAL, 11, 0, "test")
        return []
    except Exception as error:
        return [weblate_check("weblate.C024", f"Failed to use Pango: {error}")]



# Copyright © Michal Čihař <michal@weblate.org>
#
# SPDX-License-Identifier: GPL-3.0-or-later

from django.conf import settings
from django.db import models


class ContributorAgreementManager(models.Manager):
    def has_agreed(self, user, component):
        cache_key = (user.pk, component.pk)
        if cache_key not in user.cla_cache:
            user.cla_cache[cache_key] = self.filter(
                component=component, user=user
            ).exists()
        return user.cla_cache[cache_key]

    def create(self, user, component, **kwargs):
        user.cla_cache[(user.pk, component.pk)] = True
        return super().create(user=user, component=component, **kwargs)

    def order(self):
        return self.order_by("component__project__name", "component__name")


class ContributorAgreement(models.Model):
    user = models.ForeignKey(
        settings.AUTH_USER_MODEL, on_delete=models.deletion.CASCADE
    )
    component = models.ForeignKey("Component", on_delete=models.deletion.CASCADE)
    timestamp = models.DateTimeField(auto_now=True)

    objects = ContributorAgreementManager()

    class Meta:
        unique_together = [("user", "component")]
        verbose_name = "contributor agreement"
        verbose_name_plural = "contributor agreements"

    def __str__(self):
        return f"{self.user.username}:{self.component}"



# Copyright © Michal Čihař <michal@weblate.org>
#
# SPDX-License-Identifier: GPL-3.0-or-later

# Generated by Django 4.1a1 on 2022-06-29 07:18

from django.conf import settings
from django.db import migrations


def update_auditlog(apps, schema_editor):
    User = apps.get_model("weblate_auth", "User")
    AuditLog = apps.get_model("accounts", "AuditLog")
    db_alias = schema_editor.connection.alias
    logs = AuditLog.objects.using(db_alias).filter(
        user__username=settings.ANONYMOUS_USER_NAME, activity="sent-email"
    )
    for log in logs:
        try:
            log.user = User.objects.get(email=log.params["email"])
        except User.DoesNotExist:
            log.user = None
        log.save(update_fields=["user"])


class Migration(migrations.Migration):

    dependencies = [
        ("accounts", "0019_alter_auditlog_user"),
    ]

    operations = [
        migrations.RunPython(update_auditlog, migrations.RunPython.noop, elidable=True),
    ]



# Copyright © Michal Čihař <michal@weblate.org>
#
# SPDX-License-Identifier: GPL-3.0-or-later

# Generated by Django 3.2.5 on 2021-11-27 12:57

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("trans", "0141_alter_component_commit_pending_age"),
    ]

    operations = [
        migrations.CreateModel(
            name="ProjectToken",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("name", models.CharField(max_length=100, verbose_name="Name")),
                (
                    "token",
                    models.CharField(
                        default="",
                        help_text="Token value to use in API",
                        max_length=100,
                        verbose_name="Token value",
                    ),
                ),
                ("expires", models.DateTimeField(verbose_name="Expires")),
                (
                    "project",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE, to="trans.project"
                    ),
                ),
            ],
            options={
                "verbose_name": "Project Token",
                "verbose_name_plural": "Project Tokens",
                "unique_together": {("project", "token"), ("project", "name")},
            },
        ),
    ]



# Copyright © Michal Čihař <michal@weblate.org>
#
# SPDX-License-Identifier: GPL-3.0-or-later


def adjust_addon_events(apps, schema_editor, names, add, remove):
    Addon = apps.get_model("addons", "Addon")
    Event = apps.get_model("addons", "Event")
    db_alias = schema_editor.connection.alias
    for addon in Addon.objects.using(db_alias).filter(name__in=names):
        for event in add:
            Event.objects.using(db_alias).get_or_create(addon=addon, event=event)
        addon.event_set.filter(event__in=remove).delete()



# Copyright © Michal Čihař <michal@weblate.org>
#
# SPDX-License-Identifier: GPL-3.0-or-later

# Generated by Django 4.0.6 on 2022-11-29 18:42

from django.conf import settings
from django.db import migrations, models

import weblate.utils.validators


class Migration(migrations.Migration):

    dependencies = [
        ("trans", "0160_alter_change_index_together"),
    ]

    operations = [
        migrations.AlterField(
            model_name="project",
            name="name",
            field=models.CharField(
                help_text="Display name",
                max_length=60,
                unique=True,
                validators=[weblate.utils.validators.validate_project_name],
                verbose_name="Project name",
            ),
        ),
        migrations.AlterField(
            model_name="project",
            name="web",
            field=models.URLField(
                help_text="Main website of translated project.",
                validators=[weblate.utils.validators.validate_project_web],
                verbose_name="Project website",
                blank=not settings.WEBSITE_REQUIRED,
            ),
        ),
    ]



# Copyright © Michal Čihař <michal@weblate.org>
#
# SPDX-License-Identifier: GPL-3.0-or-later

"""Tests for changes browsing."""

from django.urls import reverse

from weblate.trans.models import Unit
from weblate.trans.tests.test_views import ViewTestCase


class ChangesTest(ViewTestCase):
    def test_basic(self):
        response = self.client.get(reverse("changes"))
        self.assertContains(response, "Resource update")

    def test_basic_csv_denied(self):
        response = self.client.get(reverse("changes-csv"))
        self.assertEqual(response.status_code, 403)

    def test_basic_csv(self):
        self.make_manager()
        response = self.client.get(reverse("changes-csv"))
        self.assertContains(response, "timestamp,")

    def test_filter(self):
        response = self.client.get(reverse("changes"), {"project": "test"})
        self.assertContains(response, "Resource update")
        self.assertNotContains(response, "Failed to find matching project!")
        response = self.client.get(
            reverse("changes"), {"project": "test", "component": "test"}
        )
        self.assertContains(response, "Resource update")
        self.assertNotContains(response, "Failed to find matching project!")
        response = self.client.get(
            reverse("changes"), {"project": "test", "component": "test", "lang": "cs"}
        )
        self.assertContains(response, "Resource update")
        self.assertNotContains(response, "Failed to find matching project!")
        response = self.client.get(reverse("changes"), {"lang": "cs"})
        self.assertContains(response, "Resource update")
        self.assertNotContains(response, "Failed to find matching language!")
        response = self.client.get(
            reverse("changes"), {"project": "testx", "component": "test", "lang": "cs"}
        )
        self.assertContains(response, "Resource update")
        self.assertContains(response, "Failed to find matching project!")
        response = self.client.get(
            reverse("changes"),
            {"project": "\000testx", "component": "test", "lang": "cs"},
        )
        self.assertContains(response, "Resource update")
        self.assertContains(response, "testx is not one of the available choices")

    def test_string(self):
        response = self.client.get(
            reverse("changes"), {"string": Unit.objects.first().pk}
        )
        self.assertContains(response, "New source string")
        self.assertContains(response, "Changes of string in")

    def test_user(self):
        self.edit_unit("Hello, world!\n", "Nazdar svete!\n")
        response = self.client.get(reverse("changes"), {"user": self.user.username})
        self.assertContains(response, "New translation")
        self.assertNotContains(response, "Invalid search string!")



# Copyright © Michal Čihař <michal@weblate.org>
#
# SPDX-License-Identifier: GPL-3.0-or-later

from django.db.models import CharField

from weblate.utils.validators import validate_re


class RegexField(CharField):
    default_validators = [validate_re]



# Copyright © Michal Čihař <michal@weblate.org>
#
# SPDX-License-Identifier: GPL-3.0-or-later

"""License data definitions.

This is an automatically generated file, see scripts/generate-license-data
"""

LICENSES = (
    ("Glide", "3dfx Glide License", "https://spdx.org/licenses/Glide.html", False),
    ("Abstyles", "Abstyles License", "https://spdx.org/licenses/Abstyles.html", False),
    (
        "AFL-1.1",
        "Academic Free License v1.1",
        "https://spdx.org/licenses/AFL-1.1.html",
        True,
    ),
    (
        "AFL-1.2",
        "Academic Free License v1.2",
        "https://spdx.org/licenses/AFL-1.2.html",
        True,
    ),
    (
        "AFL-2.0",
        "Academic Free License v2.0",
        "https://spdx.org/licenses/AFL-2.0.html",
        True,
    ),
    (
        "AFL-2.1",
        "Academic Free License v2.1",
        "https://spdx.org/licenses/AFL-2.1.html",
        True,
    ),
    (
        "AFL-3.0",
        "Academic Free License v3.0",
        "https://spdx.org/licenses/AFL-3.0.html",
        True,
    ),
    (
        "AMPAS",
        "Academy of Motion Picture Arts and Sciences BSD",
        "https://spdx.org/licenses/AMPAS.html",
        False,
    ),
    (
        "APL-1.0",
        "Adaptive Public License 1.0",
        "https://spdx.org/licenses/APL-1.0.html",
        True,
    ),
    (
        "Adobe-Glyph",
        "Adobe Glyph List License",
        "https://spdx.org/licenses/Adobe-Glyph.html",
        False,
    ),
    (
        "APAFML",
        "Adobe Postscript AFM License",
        "https://spdx.org/licenses/APAFML.html",
        False,
    ),
    (
        "Adobe-2006",
        "Adobe Systems Incorporated Source Code License Agreement",
        "https://spdx.org/licenses/Adobe-2006.html",
        False,
    ),
    (
        "AGPL-1.0-only",
        "Affero General Public License v1.0 only",
        "https://spdx.org/licenses/AGPL-1.0-only.html",
        False,
    ),
    (
        "AGPL-1.0-or-later",
        "Affero General Public License v1.0 or later",
        "https://spdx.org/licenses/AGPL-1.0-or-later.html",
        False,
    ),
    ("Afmparse", "Afmparse License", "https://spdx.org/licenses/Afmparse.html", False),
    (
        "Aladdin",
        "Aladdin Free Public License",
        "https://spdx.org/licenses/Aladdin.html",
        False,
    ),
    (
        "ADSL",
        "Amazon Digital Services License",
        "https://spdx.org/licenses/ADSL.html",
        False,
    ),
    (
        "AMDPLPA",
        "AMD's plpa_map.c License",
        "https://spdx.org/licenses/AMDPLPA.html",
        False,
    ),
    (
        "ANTLR-PD",
        "ANTLR Software Rights Notice",
        "https://spdx.org/licenses/ANTLR-PD.html",
        False,
    ),
    (
        "ANTLR-PD-fallback",
        "ANTLR Software Rights Notice with license fallback",
        "https://spdx.org/licenses/ANTLR-PD-fallback.html",
        False,
    ),
    (
        "Apache-1.0",
        "Apache License 1.0",
        "https://spdx.org/licenses/Apache-1.0.html",
        True,
    ),
    (
        "Apache-1.1",
        "Apache License 1.1",
        "https://spdx.org/licenses/Apache-1.1.html",
        True,
    ),
    (
        "Apache-2.0",
        "Apache License 2.0",
        "https://spdx.org/licenses/Apache-2.0.html",
        True,
    ),
    ("App-s2p", "App::s2p License", "https://spdx.org/licenses/App-s2p.html", False),
    ("AML", "Apple MIT License", "https://spdx.org/licenses/AML.html", False),
    (
        "APSL-1.0",
        "Apple Public Source License 1.0",
        "https://spdx.org/licenses/APSL-1.0.html",
        True,
    ),
    (
        "APSL-1.1",
        "Apple Public Source License 1.1",
        "https://spdx.org/licenses/APSL-1.1.html",
        True,
    ),
    (
        "APSL-1.2",
        "Apple Public Source License 1.2",
        "https://spdx.org/licenses/APSL-1.2.html",
        True,
    ),
    (
        "APSL-2.0",
        "Apple Public Source License 2.0",
        "https://spdx.org/licenses/APSL-2.0.html",
        True,
    ),
    (
        "Arphic-1999",
        "Arphic Public License",
        "https://spdx.org/licenses/Arphic-1999.html",
        False,
    ),
    (
        "Artistic-1.0",
        "Artistic License 1.0",
        "https://spdx.org/licenses/Artistic-1.0.html",
        True,
    ),
    (
        "Artistic-1.0-Perl",
        "Artistic License 1.0 (Perl)",
        "https://spdx.org/licenses/Artistic-1.0-Perl.html",
        True,
    ),
    (
        "Artistic-1.0-cl8",
        "Artistic License 1.0 w/clause 8",
        "https://spdx.org/licenses/Artistic-1.0-cl8.html",
        True,
    ),
    (
        "Artistic-2.0",
        "Artistic License 2.0",
        "https://spdx.org/licenses/Artistic-2.0.html",
        True,
    ),
    (
        "AAL",
        "Attribution Assurance License",
        "https://spdx.org/licenses/AAL.html",
        True,
    ),
    ("Baekmuk", "Baekmuk License", "https://spdx.org/licenses/Baekmuk.html", False),
    ("Bahyph", "Bahyph License", "https://spdx.org/licenses/Bahyph.html", False),
    ("Barr", "Barr License", "https://spdx.org/licenses/Barr.html", False),
    ("Beerware", "Beerware License", "https://spdx.org/licenses/Beerware.html", True),
    (
        "Bitstream-Charter",
        "Bitstream Charter Font License",
        "https://spdx.org/licenses/Bitstream-Charter.html",
        False,
    ),
    (
        "Bitstream-Vera",
        "Bitstream Vera Font License",
        "https://spdx.org/licenses/Bitstream-Vera.html",
        False,
    ),
    (
        "BitTorrent-1.0",
        "BitTorrent Open Source License v1.0",
        "https://spdx.org/licenses/BitTorrent-1.0.html",
        False,
    ),
    (
        "BitTorrent-1.1",
        "BitTorrent Open Source License v1.1",
        "https://spdx.org/licenses/BitTorrent-1.1.html",
        True,
    ),
    (
        "BlueOak-1.0.0",
        "Blue Oak Model License 1.0.0",
        "https://spdx.org/licenses/BlueOak-1.0.0.html",
        False,
    ),
    (
        "BSL-1.0",
        "Boost Software License 1.0",
        "https://spdx.org/licenses/BSL-1.0.html",
        True,
    ),
    ("Borceux", "Borceux license", "https://spdx.org/licenses/Borceux.html", False),
    (
        "BSD-1-Clause",
        "BSD 1-Clause License",
        "https://spdx.org/licenses/BSD-1-Clause.html",
        True,
    ),
    (
        "BSD-2-Clause",
        'BSD 2-Clause "Simplified" License',
        "https://spdx.org/licenses/BSD-2-Clause.html",
        True,
    ),
    (
        "BSD-2-Clause-Views",
        "BSD 2-Clause with views sentence",
        "https://spdx.org/licenses/BSD-2-Clause-Views.html",
        False,
    ),
    (
        "BSD-3-Clause",
        'BSD 3-Clause "New" or "Revised" License',
        "https://spdx.org/licenses/BSD-3-Clause.html",
        True,
    ),
    (
        "BSD-3-Clause-Clear",
        "BSD 3-Clause Clear License",
        "https://spdx.org/licenses/BSD-3-Clause-Clear.html",
        True,
    ),
    (
        "BSD-3-Clause-Modification",
        "BSD 3-Clause Modification",
        "https://spdx.org/licenses/BSD-3-Clause-Modification.html",
        False,
    ),
    (
        "BSD-3-Clause-No-Military-License",
        "BSD 3-Clause No Military License",
        "https://spdx.org/licenses/BSD-3-Clause-No-Military-License.html",
        False,
    ),
    (
        "BSD-3-Clause-No-Nuclear-License",
        "BSD 3-Clause No Nuclear License",
        "https://spdx.org/licenses/BSD-3-Clause-No-Nuclear-License.html",
        False,
    ),
    (
        "BSD-3-Clause-No-Nuclear-License-2014",
        "BSD 3-Clause No Nuclear License 2014",
        "https://spdx.org/licenses/BSD-3-Clause-No-Nuclear-License-2014.html",
        False,
    ),
    (
        "BSD-3-Clause-No-Nuclear-Warranty",
        "BSD 3-Clause No Nuclear Warranty",
        "https://spdx.org/licenses/BSD-3-Clause-No-Nuclear-Warranty.html",
        False,
    ),
    (
        "BSD-3-Clause-Open-MPI",
        "BSD 3-Clause Open MPI variant",
        "https://spdx.org/licenses/BSD-3-Clause-Open-MPI.html",
        False,
    ),
    (
        "BSD-4-Clause-Shortened",
        "BSD 4 Clause Shortened",
        "https://spdx.org/licenses/BSD-4-Clause-Shortened.html",
        False,
    ),
    (
        "BSD-4-Clause",
        'BSD 4-Clause "Original" or "Old" License',
        "https://spdx.org/licenses/BSD-4-Clause.html",
        True,
    ),
    (
        "BSD-Protection",
        "BSD Protection License",
        "https://spdx.org/licenses/BSD-Protection.html",
        False,
    ),
    (
        "BSD-Source-Code",
        "BSD Source Code Attribution",
        "https://spdx.org/licenses/BSD-Source-Code.html",
        False,
    ),
    (
        "BSD-3-Clause-Attribution",
        "BSD with attribution",
        "https://spdx.org/licenses/BSD-3-Clause-Attribution.html",
        False,
    ),
    ("0BSD", "BSD Zero Clause License", "https://spdx.org/licenses/0BSD.html", True),
    (
        "BSD-2-Clause-Patent",
        "BSD-2-Clause Plus Patent License",
        "https://spdx.org/licenses/BSD-2-Clause-Patent.html",
        True,
    ),
    (
        "BSD-4-Clause-UC",
        "BSD-4-Clause (University of California-Specific)",
        "https://spdx.org/licenses/BSD-4-Clause-UC.html",
        False,
    ),
    (
        "BUSL-1.1",
        "Business Source License 1.1",
        "https://spdx.org/licenses/BUSL-1.1.html",
        False,
    ),
    (
        "bzip2-1.0.6",
        "bzip2 and libbzip2 License v1.0.6",
        "https://spdx.org/licenses/bzip2-1.0.6.html",
        False,
    ),
    ("Caldera", "Caldera License", "https://spdx.org/licenses/Caldera.html", False),
    (
        "CECILL-1.0",
        "CeCILL Free Software License Agreement v1.0",
        "https://spdx.org/licenses/CECILL-1.0.html",
        False,
    ),
    (
        "CECILL-1.1",
        "CeCILL Free Software License Agreement v1.1",
        "https://spdx.org/licenses/CECILL-1.1.html",
        False,
    ),
    (
        "CECILL-2.0",
        "CeCILL Free Software License Agreement v2.0",
        "https://spdx.org/licenses/CECILL-2.0.html",
        True,
    ),
    (
        "CECILL-2.1",
        "CeCILL Free Software License Agreement v2.1",
        "https://spdx.org/licenses/CECILL-2.1.html",
        True,
    ),
    (
        "CECILL-B",
        "CeCILL-B Free Software License Agreement",
        "https://spdx.org/licenses/CECILL-B.html",
        True,
    ),
    (
        "CECILL-C",
        "CeCILL-C Free Software License Agreement",
        "https://spdx.org/licenses/CECILL-C.html",
        True,
    ),
    (
        "CERN-OHL-1.1",
        "CERN Open Hardware Licence v1.1",
        "https://spdx.org/licenses/CERN-OHL-1.1.html",
        False,
    ),
    (
        "CERN-OHL-1.2",
        "CERN Open Hardware Licence v1.2",
        "https://spdx.org/licenses/CERN-OHL-1.2.html",
        False,
    ),
    (
        "CERN-OHL-P-2.0",
        "CERN Open Hardware Licence Version 2 - Permissive",
        "https://spdx.org/licenses/CERN-OHL-P-2.0.html",
        True,
    ),
    (
        "CERN-OHL-S-2.0",
        "CERN Open Hardware Licence Version 2 - Strongly Reciprocal",
        "https://spdx.org/licenses/CERN-OHL-S-2.0.html",
        True,
    ),
    (
        "CERN-OHL-W-2.0",
        "CERN Open Hardware Licence Version 2 - Weakly Reciprocal",
        "https://spdx.org/licenses/CERN-OHL-W-2.0.html",
        True,
    ),
    ("checkmk", "Checkmk License", "https://spdx.org/licenses/checkmk.html", False),
    (
        "ClArtistic",
        "Clarified Artistic License",
        "https://spdx.org/licenses/ClArtistic.html",
        True,
    ),
    ("MIT-CMU", "CMU License", "https://spdx.org/licenses/MIT-CMU.html", False),
    (
        "CNRI-Jython",
        "CNRI Jython License",
        "https://spdx.org/licenses/CNRI-Jython.html",
        False,
    ),
    (
        "CNRI-Python",
        "CNRI Python License",
        "https://spdx.org/licenses/CNRI-Python.html",
        True,
    ),
    (
        "CNRI-Python-GPL-Compatible",
        "CNRI Python Open Source GPL Compatible License Agreement",
        "https://spdx.org/licenses/CNRI-Python-GPL-Compatible.html",
        False,
    ),
    (
        "CPOL-1.02",
        "Code Project Open License 1.02",
        "https://spdx.org/licenses/CPOL-1.02.html",
        False,
    ),
    (
        "CDDL-1.0",
        "Common Development and Distribution License 1.0",
        "https://spdx.org/licenses/CDDL-1.0.html",
        True,
    ),
    (
        "CDDL-1.1",
        "Common Development and Distribution License 1.1",
        "https://spdx.org/licenses/CDDL-1.1.html",
        False,
    ),
    (
        "CDL-1.0",
        "Common Documentation License 1.0",
        "https://spdx.org/licenses/CDL-1.0.html",
        False,
    ),
    ("LOOP", "Common Lisp LOOP License", "https://spdx.org/licenses/LOOP.html", False),
    (
        "CPAL-1.0",
        "Common Public Attribution License 1.0",
        "https://spdx.org/licenses/CPAL-1.0.html",
        True,
    ),
    (
        "CPL-1.0",
        "Common Public License 1.0",
        "https://spdx.org/licenses/CPL-1.0.html",
        True,
    ),
    (
        "CDLA-Permissive-1.0",
        "Community Data License Agreement Permissive 1.0",
        "https://spdx.org/licenses/CDLA-Permissive-1.0.html",
        False,
    ),
    (
        "CDLA-Permissive-2.0",
        "Community Data License Agreement Permissive 2.0",
        "https://spdx.org/licenses/CDLA-Permissive-2.0.html",
        False,
    ),
    (
        "CDLA-Sharing-1.0",
        "Community Data License Agreement Sharing 1.0",
        "https://spdx.org/licenses/CDLA-Sharing-1.0.html",
        False,
    ),
    (
        "Community-Spec-1.0",
        "Community Specification License 1.0",
        "https://spdx.org/licenses/Community-Spec-1.0.html",
        False,
    ),
    (
        "C-UDA-1.0",
        "Computational Use of Data Agreement v1.0",
        "https://spdx.org/licenses/C-UDA-1.0.html",
        False,
    ),
    (
        "CATOSL-1.1",
        "Computer Associates Trusted Open Source License 1.1",
        "https://spdx.org/licenses/CATOSL-1.1.html",
        True,
    ),
    (
        "Condor-1.1",
        "Condor Public License v1.1",
        "https://spdx.org/licenses/Condor-1.1.html",
        True,
    ),
    (
        "COIL-1.0",
        "Copyfree Open Innovation License",
        "https://spdx.org/licenses/COIL-1.0.html",
        False,
    ),
    (
        "copyleft-next-0.3.0",
        "copyleft-next 0.3.0",
        "https://spdx.org/licenses/copyleft-next-0.3.0.html",
        False,
    ),
    (
        "copyleft-next-0.3.1",
        "copyleft-next 0.3.1",
        "https://spdx.org/licenses/copyleft-next-0.3.1.html",
        False,
    ),
    (
        "CC-BY-1.0",
        "Creative Commons Attribution 1.0 Generic",
        "https://spdx.org/licenses/CC-BY-1.0.html",
        False,
    ),
    (
        "CC-BY-2.0",
        "Creative Commons Attribution 2.0 Generic",
        "https://spdx.org/licenses/CC-BY-2.0.html",
        False,
    ),
    (
        "CC-BY-2.5-AU",
        "Creative Commons Attribution 2.5 Australia",
        "https://spdx.org/licenses/CC-BY-2.5-AU.html",
        False,
    ),
    (
        "CC-BY-2.5",
        "Creative Commons Attribution 2.5 Generic",
        "https://spdx.org/licenses/CC-BY-2.5.html",
        False,
    ),
    (
        "CC-BY-3.0-AT",
        "Creative Commons Attribution 3.0 Austria",
        "https://spdx.org/licenses/CC-BY-3.0-AT.html",
        False,
    ),
    (
        "CC-BY-3.0-DE",
        "Creative Commons Attribution 3.0 Germany",
        "https://spdx.org/licenses/CC-BY-3.0-DE.html",
        False,
    ),
    (
        "CC-BY-3.0-IGO",
        "Creative Commons Attribution 3.0 IGO",
        "https://spdx.org/licenses/CC-BY-3.0-IGO.html",
        False,
    ),
    (
        "CC-BY-3.0-NL",
        "Creative Commons Attribution 3.0 Netherlands",
        "https://spdx.org/licenses/CC-BY-3.0-NL.html",
        False,
    ),
    (
        "CC-BY-3.0-US",
        "Creative Commons Attribution 3.0 United States",
        "https://spdx.org/licenses/CC-BY-3.0-US.html",
        False,
    ),
    (
        "CC-BY-3.0",
        "Creative Commons Attribution 3.0 Unported",
        "https://spdx.org/licenses/CC-BY-3.0.html",
        True,
    ),
    (
        "CC-BY-4.0",
        "Creative Commons Attribution 4.0 International",
        "https://spdx.org/licenses/CC-BY-4.0.html",
        True,
    ),
    (
        "CC-BY-ND-1.0",
        "Creative Commons Attribution No Derivatives 1.0 Generic",
        "https://spdx.org/licenses/CC-BY-ND-1.0.html",
        False,
    ),
    (
        "CC-BY-ND-2.0",
        "Creative Commons Attribution No Derivatives 2.0 Generic",
        "https://spdx.org/licenses/CC-BY-ND-2.0.html",
        False,
    ),
    (
        "CC-BY-ND-2.5",
        "Creative Commons Attribution No Derivatives 2.5 Generic",
        "https://spdx.org/licenses/CC-BY-ND-2.5.html",
        False,
    ),
    (
        "CC-BY-ND-3.0-DE",
        "Creative Commons Attribution No Derivatives 3.0 Germany",
        "https://spdx.org/licenses/CC-BY-ND-3.0-DE.html",
        False,
    ),
    (
        "CC-BY-ND-3.0",
        "Creative Commons Attribution No Derivatives 3.0 Unported",
        "https://spdx.org/licenses/CC-BY-ND-3.0.html",
        False,
    ),
    (
        "CC-BY-ND-4.0",
        "Creative Commons Attribution No Derivatives 4.0 International",
        "https://spdx.org/licenses/CC-BY-ND-4.0.html",
        False,
    ),
    (
        "CC-BY-NC-1.0",
        "Creative Commons Attribution Non Commercial 1.0 Generic",
        "https://spdx.org/licenses/CC-BY-NC-1.0.html",
        False,
    ),
    (
        "CC-BY-NC-2.0",
        "Creative Commons Attribution Non Commercial 2.0 Generic",
        "https://spdx.org/licenses/CC-BY-NC-2.0.html",
        False,
    ),
    (
        "CC-BY-NC-2.5",
        "Creative Commons Attribution Non Commercial 2.5 Generic",
        "https://spdx.org/licenses/CC-BY-NC-2.5.html",
        False,
    ),
    (
        "CC-BY-NC-3.0-DE",
        "Creative Commons Attribution Non Commercial 3.0 Germany",
        "https://spdx.org/licenses/CC-BY-NC-3.0-DE.html",
        False,
    ),
    (
        "CC-BY-NC-3.0",
        "Creative Commons Attribution Non Commercial 3.0 Unported",
        "https://spdx.org/licenses/CC-BY-NC-3.0.html",
        False,
    ),
    (
        "CC-BY-NC-4.0",
        "Creative Commons Attribution Non Commercial 4.0 International",
        "https://spdx.org/licenses/CC-BY-NC-4.0.html",
        False,
    ),
    (
        "CC-BY-NC-ND-1.0",
        "Creative Commons Attribution Non Commercial No Derivatives 1.0 Generic",
        "https://spdx.org/licenses/CC-BY-NC-ND-1.0.html",
        False,
    ),
    (
        "CC-BY-NC-ND-2.0",
        "Creative Commons Attribution Non Commercial No Derivatives 2.0 Generic",
        "https://spdx.org/licenses/CC-BY-NC-ND-2.0.html",
        False,
    ),
    (
        "CC-BY-NC-ND-2.5",
        "Creative Commons Attribution Non Commercial No Derivatives 2.5 Generic",
        "https://spdx.org/licenses/CC-BY-NC-ND-2.5.html",
        False,
    ),
    (
        "CC-BY-NC-ND-3.0-DE",
        "Creative Commons Attribution Non Commercial No Derivatives 3.0 Germany",
        "https://spdx.org/licenses/CC-BY-NC-ND-3.0-DE.html",
        False,
    ),
    (
        "CC-BY-NC-ND-3.0-IGO",
        "Creative Commons Attribution Non Commercial No Derivatives 3.0 IGO",
        "https://spdx.org/licenses/CC-BY-NC-ND-3.0-IGO.html",
        False,
    ),
    (
        "CC-BY-NC-ND-3.0",
        "Creative Commons Attribution Non Commercial No Derivatives 3.0 Unported",
        "https://spdx.org/licenses/CC-BY-NC-ND-3.0.html",
        False,
    ),
    (
        "CC-BY-NC-ND-4.0",
        "Creative Commons Attribution Non Commercial No Derivatives 4.0 International",
        "https://spdx.org/licenses/CC-BY-NC-ND-4.0.html",
        False,
    ),
    (
        "CC-BY-NC-SA-1.0",
        "Creative Commons Attribution Non Commercial Share Alike 1.0 Generic",
        "https://spdx.org/licenses/CC-BY-NC-SA-1.0.html",
        False,
    ),
    (
        "CC-BY-NC-SA-2.0-UK",
        "Creative Commons Attribution Non Commercial Share Alike 2.0 England and Wales",
        "https://spdx.org/licenses/CC-BY-NC-SA-2.0-UK.html",
        False,
    ),
    (
        "CC-BY-NC-SA-2.0",
        "Creative Commons Attribution Non Commercial Share Alike 2.0 Generic",
        "https://spdx.org/licenses/CC-BY-NC-SA-2.0.html",
        False,
    ),
    (
        "CC-BY-NC-SA-2.0-DE",
        "Creative Commons Attribution Non Commercial Share Alike 2.0 Germany",
        "https://spdx.org/licenses/CC-BY-NC-SA-2.0-DE.html",
        False,
    ),
    (
        "CC-BY-NC-SA-2.5",
        "Creative Commons Attribution Non Commercial Share Alike 2.5 Generic",
        "https://spdx.org/licenses/CC-BY-NC-SA-2.5.html",
        False,
    ),
    (
        "CC-BY-NC-SA-3.0-DE",
        "Creative Commons Attribution Non Commercial Share Alike 3.0 Germany",
        "https://spdx.org/licenses/CC-BY-NC-SA-3.0-DE.html",
        False,
    ),
    (
        "CC-BY-NC-SA-3.0-IGO",
        "Creative Commons Attribution Non Commercial Share Alike 3.0 IGO",
        "https://spdx.org/licenses/CC-BY-NC-SA-3.0-IGO.html",
        False,
    ),
    (
        "CC-BY-NC-SA-3.0",
        "Creative Commons Attribution Non Commercial Share Alike 3.0 Unported",
        "https://spdx.org/licenses/CC-BY-NC-SA-3.0.html",
        False,
    ),
    (
        "CC-BY-NC-SA-4.0",
        "Creative Commons Attribution Non Commercial Share Alike 4.0 International",
        "https://spdx.org/licenses/CC-BY-NC-SA-4.0.html",
        False,
    ),
    (
        "CC-BY-SA-1.0",
        "Creative Commons Attribution Share Alike 1.0 Generic",
        "https://spdx.org/licenses/CC-BY-SA-1.0.html",
        False,
    ),
    (
        "CC-BY-SA-2.0-UK",
        "Creative Commons Attribution Share Alike 2.0 England and Wales",
        "https://spdx.org/licenses/CC-BY-SA-2.0-UK.html",
        False,
    ),
    (
        "CC-BY-SA-2.0",
        "Creative Commons Attribution Share Alike 2.0 Generic",
        "https://spdx.org/licenses/CC-BY-SA-2.0.html",
        False,
    ),
    (
        "CC-BY-SA-2.1-JP",
        "Creative Commons Attribution Share Alike 2.1 Japan",
        "https://spdx.org/licenses/CC-BY-SA-2.1-JP.html",
        False,
    ),
    (
        "CC-BY-SA-2.5",
        "Creative Commons Attribution Share Alike 2.5 Generic",
        "https://spdx.org/licenses/CC-BY-SA-2.5.html",
        False,
    ),
    (
        "CC-BY-SA-3.0-AT",
        "Creative Commons Attribution Share Alike 3.0 Austria",
        "https://spdx.org/licenses/CC-BY-SA-3.0-AT.html",
        False,
    ),
    (
        "CC-BY-SA-3.0-DE",
        "Creative Commons Attribution Share Alike 3.0 Germany",
        "https://spdx.org/licenses/CC-BY-SA-3.0-DE.html",
        False,
    ),
    (
        "CC-BY-SA-3.0",
        "Creative Commons Attribution Share Alike 3.0 Unported",
        "https://spdx.org/licenses/CC-BY-SA-3.0.html",
        True,
    ),
    (
        "CC-BY-SA-4.0",
        "Creative Commons Attribution Share Alike 4.0 International",
        "https://spdx.org/licenses/CC-BY-SA-4.0.html",
        True,
    ),
    (
        "CC-BY-NC-SA-2.0-FR",
        "Creative Commons Attribution-NonCommercial-ShareAlike 2.0 France",
        "https://spdx.org/licenses/CC-BY-NC-SA-2.0-FR.html",
        False,
    ),
    (
        "CC-PDDC",
        "Creative Commons Public Domain Dedication and Certification",
        "https://spdx.org/licenses/CC-PDDC.html",
        False,
    ),
    (
        "CC0-1.0",
        "Creative Commons Zero v1.0 Universal",
        "https://spdx.org/licenses/CC0-1.0.html",
        True,
    ),
    (
        "Crossword",
        "Crossword License",
        "https://spdx.org/licenses/Crossword.html",
        False,
    ),
    (
        "CAL-1.0",
        "Cryptographic Autonomy License 1.0",
        "https://spdx.org/licenses/CAL-1.0.html",
        True,
    ),
    (
        "CAL-1.0-Combined-Work-Exception",
        "Cryptographic Autonomy License 1.0 (Combined Work Exception)",
        "https://spdx.org/licenses/CAL-1.0-Combined-Work-Exception.html",
        True,
    ),
    (
        "CrystalStacker",
        "CrystalStacker License",
        "https://spdx.org/licenses/CrystalStacker.html",
        False,
    ),
    (
        "CUA-OPL-1.0",
        "CUA Office Public License v1.0",
        "https://spdx.org/licenses/CUA-OPL-1.0.html",
        True,
    ),
    ("Cube", "Cube License", "https://spdx.org/licenses/Cube.html", False),
    ("curl", "curl License", "https://spdx.org/licenses/curl.html", False),
    (
        "DL-DE-BY-2.0",
        "Data licence Germany – attribution – version 2.0",
        "https://spdx.org/licenses/DL-DE-BY-2.0.html",
        False,
    ),
    (
        "DRL-1.0",
        "Detection Rule License 1.0",
        "https://spdx.org/licenses/DRL-1.0.html",
        False,
    ),
    (
        "D-FSL-1.0",
        "Deutsche Freie Software Lizenz",
        "https://spdx.org/licenses/D-FSL-1.0.html",
        False,
    ),
    ("diffmark", "diffmark license", "https://spdx.org/licenses/diffmark.html", False),
    (
        "WTFPL",
        "Do What The F*ck You Want To Public License",
        "https://spdx.org/licenses/WTFPL.html",
        True,
    ),
    ("DOC", "DOC License", "https://spdx.org/licenses/DOC.html", False),
    ("Dotseqn", "Dotseqn License", "https://spdx.org/licenses/Dotseqn.html", False),
    ("DSDP", "DSDP License", "https://spdx.org/licenses/DSDP.html", False),
    ("dvipdfm", "dvipdfm License", "https://spdx.org/licenses/dvipdfm.html", False),
    (
        "EPL-1.0",
        "Eclipse Public License 1.0",
        "https://spdx.org/licenses/EPL-1.0.html",
        True,
    ),
    (
        "EPL-2.0",
        "Eclipse Public License 2.0",
        "https://spdx.org/licenses/EPL-2.0.html",
        True,
    ),
    (
        "ECL-1.0",
        "Educational Community License v1.0",
        "https://spdx.org/licenses/ECL-1.0.html",
        True,
    ),
    (
        "ECL-2.0",
        "Educational Community License v2.0",
        "https://spdx.org/licenses/ECL-2.0.html",
        True,
    ),
    (
        "eGenix",
        "eGenix.com Public License 1.1.0",
        "https://spdx.org/licenses/eGenix.html",
        False,
    ),
    (
        "EFL-1.0",
        "Eiffel Forum License v1.0",
        "https://spdx.org/licenses/EFL-1.0.html",
        True,
    ),
    (
        "EFL-2.0",
        "Eiffel Forum License v2.0",
        "https://spdx.org/licenses/EFL-2.0.html",
        True,
    ),
    (
        "Elastic-2.0",
        "Elastic License 2.0",
        "https://spdx.org/licenses/Elastic-2.0.html",
        False,
    ),
    (
        "MIT-advertising",
        "Enlightenment License (e16)",
        "https://spdx.org/licenses/MIT-advertising.html",
        False,
    ),
    ("MIT-enna", "enna License", "https://spdx.org/licenses/MIT-enna.html", False),
    (
        "Entessa",
        "Entessa Public License v1.0",
        "https://spdx.org/licenses/Entessa.html",
        True,
    ),
    ("EPICS", "EPICS Open License", "https://spdx.org/licenses/EPICS.html", False),
    (
        "ErlPL-1.1",
        "Erlang Public License v1.1",
        "https://spdx.org/licenses/ErlPL-1.1.html",
        False,
    ),
    (
        "etalab-2.0",
        "Etalab Open License 2.0",
        "https://spdx.org/licenses/etalab-2.0.html",
        False,
    ),
    (
        "EUDatagrid",
        "EU DataGrid Software License",
        "https://spdx.org/licenses/EUDatagrid.html",
        True,
    ),
    (
        "EUPL-1.0",
        "European Union Public License 1.0",
        "https://spdx.org/licenses/EUPL-1.0.html",
        False,
    ),
    (
        "EUPL-1.1",
        "European Union Public License 1.1",
        "https://spdx.org/licenses/EUPL-1.1.html",
        True,
    ),
    (
        "EUPL-1.2",
        "European Union Public License 1.2",
        "https://spdx.org/licenses/EUPL-1.2.html",
        True,
    ),
    ("Eurosym", "Eurosym License", "https://spdx.org/licenses/Eurosym.html", False),
    ("Fair", "Fair License", "https://spdx.org/licenses/Fair.html", True),
    ("MIT-feh", "feh License", "https://spdx.org/licenses/MIT-feh.html", False),
    (
        "Frameworx-1.0",
        "Frameworx Open License 1.0",
        "https://spdx.org/licenses/Frameworx-1.0.html",
        True,
    ),
    (
        "FDK-AAC",
        "Fraunhofer FDK AAC Codec Library",
        "https://spdx.org/licenses/FDK-AAC.html",
        False,
    ),
    (
        "FreeBSD-DOC",
        "FreeBSD Documentation License",
        "https://spdx.org/licenses/FreeBSD-DOC.html",
        False,
    ),
    (
        "FreeImage",
        "FreeImage Public License v1.0",
        "https://spdx.org/licenses/FreeImage.html",
        False,
    ),
    ("FTL", "Freetype Project License", "https://spdx.org/licenses/FTL.html", True),
    (
        "FSFAP",
        "FSF All Permissive License",
        "https://spdx.org/licenses/FSFAP.html",
        True,
    ),
    ("FSFUL", "FSF Unlimited License", "https://spdx.org/licenses/FSFUL.html", False),
    (
        "FSFULLRWD",
        "FSF Unlimited License (With License Retention    and Warranty Disclaimer)",
        "https://spdx.org/licenses/FSFULLRWD.html",
        False,
    ),
    (
        "FSFULLR",
        "FSF Unlimited License (with License Retention)",
        "https://spdx.org/licenses/FSFULLR.html",
        False,
    ),
    ("GD", "GD License", "https://spdx.org/licenses/GD.html", False),
    ("Giftware", "Giftware License", "https://spdx.org/licenses/Giftware.html", False),
    ("GL2PS", "GL2PS License", "https://spdx.org/licenses/GL2PS.html", False),
    ("Glulxe", "Glulxe License", "https://spdx.org/licenses/Glulxe.html", False),
    (
        "AGPL-3.0-only",
        "GNU Affero General Public License v3.0 only",
        "https://spdx.org/licenses/AGPL-3.0-only.html",
        True,
    ),
    (
        "AGPL-3.0-or-later",
        "GNU Affero General Public License v3.0 or later",
        "https://spdx.org/licenses/AGPL-3.0-or-later.html",
        True,
    ),
    (
        "GFDL-1.1-only",
        "GNU Free Documentation License v1.1 only",
        "https://spdx.org/licenses/GFDL-1.1-only.html",
        True,
    ),
    (
        "GFDL-1.1-invariants-only",
        "GNU Free Documentation License v1.1 only - invariants",
        "https://spdx.org/licenses/GFDL-1.1-invariants-only.html",
        False,
    ),
    (
        "GFDL-1.1-no-invariants-only",
        "GNU Free Documentation License v1.1 only - no invariants",
        "https://spdx.org/licenses/GFDL-1.1-no-invariants-only.html",
        False,
    ),
    (
        "GFDL-1.1-or-later",
        "GNU Free Documentation License v1.1 or later",
        "https://spdx.org/licenses/GFDL-1.1-or-later.html",
        True,
    ),
    (
        "GFDL-1.1-invariants-or-later",
        "GNU Free Documentation License v1.1 or later - invariants",
        "https://spdx.org/licenses/GFDL-1.1-invariants-or-later.html",
        False,
    ),
    (
        "GFDL-1.1-no-invariants-or-later",
        "GNU Free Documentation License v1.1 or later - no invariants",
        "https://spdx.org/licenses/GFDL-1.1-no-invariants-or-later.html",
        False,
    ),
    (
        "GFDL-1.2-only",
        "GNU Free Documentation License v1.2 only",
        "https://spdx.org/licenses/GFDL-1.2-only.html",
        True,
    ),
    (
        "GFDL-1.2-invariants-only",
        "GNU Free Documentation License v1.2 only - invariants",
        "https://spdx.org/licenses/GFDL-1.2-invariants-only.html",
        False,
    ),
    (
        "GFDL-1.2-no-invariants-only",
        "GNU Free Documentation License v1.2 only - no invariants",
        "https://spdx.org/licenses/GFDL-1.2-no-invariants-only.html",
        False,
    ),
    (
        "GFDL-1.2-or-later",
        "GNU Free Documentation License v1.2 or later",
        "https://spdx.org/licenses/GFDL-1.2-or-later.html",
        True,
    ),
    (
        "GFDL-1.2-invariants-or-later",
        "GNU Free Documentation License v1.2 or later - invariants",
        "https://spdx.org/licenses/GFDL-1.2-invariants-or-later.html",
        False,
    ),
    (
        "GFDL-1.2-no-invariants-or-later",
        "GNU Free Documentation License v1.2 or later - no invariants",
        "https://spdx.org/licenses/GFDL-1.2-no-invariants-or-later.html",
        False,
    ),
    (
        "GFDL-1.3-only",
        "GNU Free Documentation License v1.3 only",
        "https://spdx.org/licenses/GFDL-1.3-only.html",
        True,
    ),
    (
        "GFDL-1.3-invariants-only",
        "GNU Free Documentation License v1.3 only - invariants",
        "https://spdx.org/licenses/GFDL-1.3-invariants-only.html",
        False,
    ),
    (
        "GFDL-1.3-no-invariants-only",
        "GNU Free Documentation License v1.3 only - no invariants",
        "https://spdx.org/licenses/GFDL-1.3-no-invariants-only.html",
        False,
    ),
    (
        "GFDL-1.3-or-later",
        "GNU Free Documentation License v1.3 or later",
        "https://spdx.org/licenses/GFDL-1.3-or-later.html",
        True,
    ),
    (
        "GFDL-1.3-invariants-or-later",
        "GNU Free Documentation License v1.3 or later - invariants",
        "https://spdx.org/licenses/GFDL-1.3-invariants-or-later.html",
        False,
    ),
    (
        "GFDL-1.3-no-invariants-or-later",
        "GNU Free Documentation License v1.3 or later - no invariants",
        "https://spdx.org/licenses/GFDL-1.3-no-invariants-or-later.html",
        False,
    ),
    (
        "GPL-1.0-only",
        "GNU General Public License v1.0 only",
        "https://spdx.org/licenses/GPL-1.0-only.html",
        False,
    ),
    (
        "GPL-1.0-or-later",
        "GNU General Public License v1.0 or later",
        "https://spdx.org/licenses/GPL-1.0-or-later.html",
        False,
    ),
    (
        "GPL-2.0-only",
        "GNU General Public License v2.0 only",
        "https://spdx.org/licenses/GPL-2.0-only.html",
        True,
    ),
    (
        "GPL-2.0-or-later",
        "GNU General Public License v2.0 or later",
        "https://spdx.org/licenses/GPL-2.0-or-later.html",
        True,
    ),
    (
        "GPL-3.0-only",
        "GNU General Public License v3.0 only",
        "https://spdx.org/licenses/GPL-3.0-only.html",
        True,
    ),
    (
        "GPL-3.0-or-later",
        "GNU General Public License v3.0 or later",
        "https://spdx.org/licenses/GPL-3.0-or-later.html",
        True,
    ),
    (
        "LGPL-2.1-only",
        "GNU Lesser General Public License v2.1 only",
        "https://spdx.org/licenses/LGPL-2.1-only.html",
        True,
    ),
    (
        "LGPL-2.1-or-later",
        "GNU Lesser General Public License v2.1 or later",
        "https://spdx.org/licenses/LGPL-2.1-or-later.html",
        True,
    ),
    (
        "LGPL-3.0-only",
        "GNU Lesser General Public License v3.0 only",
        "https://spdx.org/licenses/LGPL-3.0-only.html",
        True,
    ),
    (
        "LGPL-3.0-or-later",
        "GNU Lesser General Public License v3.0 or later",
        "https://spdx.org/licenses/LGPL-3.0-or-later.html",
        True,
    ),
    (
        "LGPL-2.0-only",
        "GNU Library General Public License v2 only",
        "https://spdx.org/licenses/LGPL-2.0-only.html",
        True,
    ),
    (
        "LGPL-2.0-or-later",
        "GNU Library General Public License v2 or later",
        "https://spdx.org/licenses/LGPL-2.0-or-later.html",
        True,
    ),
    ("gnuplot", "gnuplot License", "https://spdx.org/licenses/gnuplot.html", True),
    (
        "GLWTPL",
        "Good Luck With That Public License",
        "https://spdx.org/licenses/GLWTPL.html",
        False,
    ),
    (
        "Graphics-Gems",
        "Graphics Gems License",
        "https://spdx.org/licenses/Graphics-Gems.html",
        False,
    ),
    (
        "gSOAP-1.3b",
        "gSOAP Public License v1.3b",
        "https://spdx.org/licenses/gSOAP-1.3b.html",
        False,
    ),
    (
        "HaskellReport",
        "Haskell Language Report License",
        "https://spdx.org/licenses/HaskellReport.html",
        False,
    ),
    (
        "Hippocratic-2.1",
        "Hippocratic License 2.1",
        "https://spdx.org/licenses/Hippocratic-2.1.html",
        False,
    ),
    (
        "HPND",
        "Historical Permission Notice and Disclaimer",
        "https://spdx.org/licenses/HPND.html",
        True,
    ),
    (
        "HPND-sell-variant",
        "Historical Permission Notice and Disclaimer - sell variant",
        "https://spdx.org/licenses/HPND-sell-variant.html",
        False,
    ),
    (
        "HPND-export-US",
        "HPND with US Government export control warning",
        "https://spdx.org/licenses/HPND-export-US.html",
        False,
    ),
    ("HTMLTIDY", "HTML Tidy License", "https://spdx.org/licenses/HTMLTIDY.html", False),
    (
        "IBM-pibs",
        "IBM PowerPC Initialization and Boot Software",
        "https://spdx.org/licenses/IBM-pibs.html",
        False,
    ),
    (
        "IPL-1.0",
        "IBM Public License v1.0",
        "https://spdx.org/licenses/IPL-1.0.html",
        True,
    ),
    ("ICU", "ICU License", "https://spdx.org/licenses/ICU.html", False),
    (
        "ImageMagick",
        "ImageMagick License",
        "https://spdx.org/licenses/ImageMagick.html",
        False,
    ),
    (
        "iMatix",
        "iMatix Standard Function Library Agreement",
        "https://spdx.org/licenses/iMatix.html",
        True,
    ),
    ("Imlib2", "Imlib2 License", "https://spdx.org/licenses/Imlib2.html", True),
    (
        "IJG",
        "Independent JPEG Group License",
        "https://spdx.org/licenses/IJG.html",
        True,
    ),
    (
        "IJG-short",
        "Independent JPEG Group License - short",
        "https://spdx.org/licenses/IJG-short.html",
        False,
    ),
    ("Info-ZIP", "Info-ZIP License", "https://spdx.org/licenses/Info-ZIP.html", False),
    (
        "Intel-ACPI",
        "Intel ACPI Software License Agreement",
        "https://spdx.org/licenses/Intel-ACPI.html",
        False,
    ),
    (
        "Intel",
        "Intel Open Source License",
        "https://spdx.org/licenses/Intel.html",
        True,
    ),
    (
        "Interbase-1.0",
        "Interbase Public License v1.0",
        "https://spdx.org/licenses/Interbase-1.0.html",
        False,
    ),
    ("IPA", "IPA Font License", "https://spdx.org/licenses/IPA.html", True),
    ("ISC", "ISC License", "https://spdx.org/licenses/ISC.html", True),
    ("Jam", "Jam License", "https://spdx.org/licenses/Jam.html", True),
    (
        "JPNIC",
        "Japan Network Information Center License",
        "https://spdx.org/licenses/JPNIC.html",
        False,
    ),
    (
        "JasPer-2.0",
        "JasPer License",
        "https://spdx.org/licenses/JasPer-2.0.html",
        False,
    ),
    ("JSON", "JSON License", "https://spdx.org/licenses/JSON.html", False),
    (
        "Knuth-CTAN",
        "Knuth CTAN License",
        "https://spdx.org/licenses/Knuth-CTAN.html",
        False,
    ),
    (
        "LPPL-1.0",
        "LaTeX Project Public License v1.0",
        "https://spdx.org/licenses/LPPL-1.0.html",
        False,
    ),
    (
        "LPPL-1.1",
        "LaTeX Project Public License v1.1",
        "https://spdx.org/licenses/LPPL-1.1.html",
        False,
    ),
    (
        "LPPL-1.2",
        "LaTeX Project Public License v1.2",
        "https://spdx.org/licenses/LPPL-1.2.html",
        True,
    ),
    (
        "LPPL-1.3a",
        "LaTeX Project Public License v1.3a",
        "https://spdx.org/licenses/LPPL-1.3a.html",
        True,
    ),
    (
        "LPPL-1.3c",
        "LaTeX Project Public License v1.3c",
        "https://spdx.org/licenses/LPPL-1.3c.html",
        True,
    ),
    ("Latex2e", "Latex2e License", "https://spdx.org/licenses/Latex2e.html", False),
    (
        "BSD-3-Clause-LBNL",
        "Lawrence Berkeley National Labs BSD variant license",
        "https://spdx.org/licenses/BSD-3-Clause-LBNL.html",
        True,
    ),
    (
        "Leptonica",
        "Leptonica License",
        "https://spdx.org/licenses/Leptonica.html",
        False,
    ),
    (
        "LGPLLR",
        "Lesser General Public License For Linguistic Resources",
        "https://spdx.org/licenses/LGPLLR.html",
        False,
    ),
    ("Libpng", "libpng License", "https://spdx.org/licenses/Libpng.html", False),
    (
        "libselinux-1.0",
        "libselinux public domain notice",
        "https://spdx.org/licenses/libselinux-1.0.html",
        False,
    ),
    ("libtiff", "libtiff License", "https://spdx.org/licenses/libtiff.html", False),
    (
        "libutil-David-Nugent",
        "libutil David Nugent License",
        "https://spdx.org/licenses/libutil-David-Nugent.html",
        False,
    ),
    (
        "LAL-1.2",
        "Licence Art Libre 1.2",
        "https://spdx.org/licenses/LAL-1.2.html",
        False,
    ),
    (
        "LAL-1.3",
        "Licence Art Libre 1.3",
        "https://spdx.org/licenses/LAL-1.3.html",
        False,
    ),
    (
        "LiLiQ-P-1.1",
        "Licence Libre du Québec – Permissive version 1.1",
        "https://spdx.org/licenses/LiLiQ-P-1.1.html",
        True,
    ),
    (
        "LiLiQ-Rplus-1.1",
        "Licence Libre du Québec – Réciprocité forte version 1.1",
        "https://spdx.org/licenses/LiLiQ-Rplus-1.1.html",
        True,
    ),
    (
        "LiLiQ-R-1.1",
        "Licence Libre du Québec – Réciprocité version 1.1",
        "https://spdx.org/licenses/LiLiQ-R-1.1.html",
        True,
    ),
    (
        "Linux-OpenIB",
        "Linux Kernel Variant of OpenIB.org license",
        "https://spdx.org/licenses/Linux-OpenIB.html",
        False,
    ),
    (
        "Linux-man-pages-copyleft",
        "Linux man-pages Copyleft",
        "https://spdx.org/licenses/Linux-man-pages-copyleft.html",
        False,
    ),
    (
        "LPL-1.02",
        "Lucent Public License v1.02",
        "https://spdx.org/licenses/LPL-1.02.html",
        True,
    ),
    (
        "LPL-1.0",
        "Lucent Public License Version 1.0",
        "https://spdx.org/licenses/LPL-1.0.html",
        True,
    ),
    (
        "LZMA-SDK-9.11-to-9.20",
        "LZMA SDK License (versions 9.11 to 9.20)",
        "https://spdx.org/licenses/LZMA-SDK-9.11-to-9.20.html",
        False,
    ),
    (
        "LZMA-SDK-9.22",
        "LZMA SDK License (versions 9.22 and beyond)",
        "https://spdx.org/licenses/LZMA-SDK-9.22.html",
        False,
    ),
    (
        "MakeIndex",
        "MakeIndex License",
        "https://spdx.org/licenses/MakeIndex.html",
        False,
    ),
    (
        "MTLL",
        "Matrix Template Library License",
        "https://spdx.org/licenses/MTLL.html",
        False,
    ),
    (
        "MS-LPL",
        "Microsoft Limited Public License",
        "https://spdx.org/licenses/MS-LPL.html",
        False,
    ),
    ("MS-PL", "Microsoft Public License", "https://spdx.org/licenses/MS-PL.html", True),
    (
        "MS-RL",
        "Microsoft Reciprocal License",
        "https://spdx.org/licenses/MS-RL.html",
        True,
    ),
    ("Minpack", "Minpack License", "https://spdx.org/licenses/Minpack.html", False),
    (
        "MITNFA",
        "MIT +no-false-attribs license",
        "https://spdx.org/licenses/MITNFA.html",
        False,
    ),
    ("MIT", "MIT License", "https://spdx.org/licenses/MIT.html", True),
    (
        "MIT-Modern-Variant",
        "MIT License Modern Variant",
        "https://spdx.org/licenses/MIT-Modern-Variant.html",
        True,
    ),
    ("MIT-0", "MIT No Attribution", "https://spdx.org/licenses/MIT-0.html", True),
    (
        "MIT-open-group",
        "MIT Open Group variant",
        "https://spdx.org/licenses/MIT-open-group.html",
        False,
    ),
    ("MIT-Wu", "MIT Tom Wu Variant", "https://spdx.org/licenses/MIT-Wu.html", False),
    ("Motosoto", "Motosoto License", "https://spdx.org/licenses/Motosoto.html", True),
    (
        "MPL-1.0",
        "Mozilla Public License 1.0",
        "https://spdx.org/licenses/MPL-1.0.html",
        True,
    ),
    (
        "MPL-1.1",
        "Mozilla Public License 1.1",
        "https://spdx.org/licenses/MPL-1.1.html",
        True,
    ),
    (
        "MPL-2.0",
        "Mozilla Public License 2.0",
        "https://spdx.org/licenses/MPL-2.0.html",
        True,
    ),
    (
        "MPL-2.0-no-copyleft-exception",
        "Mozilla Public License 2.0 (no copyleft exception)",
        "https://spdx.org/licenses/MPL-2.0-no-copyleft-exception.html",
        True,
    ),
    (
        "mpi-permissive",
        "mpi Permissive License",
        "https://spdx.org/licenses/mpi-permissive.html",
        False,
    ),
    ("mpich2", "mpich2 License", "https://spdx.org/licenses/mpich2.html", False),
    ("mplus", "mplus Font License", "https://spdx.org/licenses/mplus.html", False),
    (
        "MulanPSL-1.0",
        "Mulan Permissive Software License, Version 1",
        "https://spdx.org/licenses/MulanPSL-1.0.html",
        False,
    ),
    (
        "MulanPSL-2.0",
        "Mulan Permissive Software License, Version 2",
        "https://spdx.org/licenses/MulanPSL-2.0.html",
        True,
    ),
    ("Multics", "Multics License", "https://spdx.org/licenses/Multics.html", True),
    ("Mup", "Mup License", "https://spdx.org/licenses/Mup.html", False),
    (
        "NAIST-2003",
        "Nara Institute of Science and Technology License (2003)",
        "https://spdx.org/licenses/NAIST-2003.html",
        False,
    ),
    (
        "NASA-1.3",
        "NASA Open Source Agreement 1.3",
        "https://spdx.org/licenses/NASA-1.3.html",
        True,
    ),
    ("Naumen", "Naumen Public License", "https://spdx.org/licenses/Naumen.html", True),
    (
        "NBPL-1.0",
        "Net Boolean Public License v1",
        "https://spdx.org/licenses/NBPL-1.0.html",
        False,
    ),
    ("Net-SNMP", "Net-SNMP License", "https://spdx.org/licenses/Net-SNMP.html", False),
    ("NetCDF", "NetCDF license", "https://spdx.org/licenses/NetCDF.html", False),
    (
        "NGPL",
        "Nethack General Public License",
        "https://spdx.org/licenses/NGPL.html",
        True,
    ),
    (
        "NOSL",
        "Netizen Open Source License",
        "https://spdx.org/licenses/NOSL.html",
        True,
    ),
    (
        "NPL-1.0",
        "Netscape Public License v1.0",
        "https://spdx.org/licenses/NPL-1.0.html",
        True,
    ),
    (
        "NPL-1.1",
        "Netscape Public License v1.1",
        "https://spdx.org/licenses/NPL-1.1.html",
        True,
    ),
    ("Newsletr", "Newsletr License", "https://spdx.org/licenses/Newsletr.html", False),
    (
        "NICTA-1.0",
        "NICTA Public Software License, Version 1.0",
        "https://spdx.org/licenses/NICTA-1.0.html",
        False,
    ),
    (
        "NIST-PD",
        "NIST Public Domain Notice",
        "https://spdx.org/licenses/NIST-PD.html",
        False,
    ),
    (
        "NIST-PD-fallback",
        "NIST Public Domain Notice with license fallback",
        "https://spdx.org/licenses/NIST-PD-fallback.html",
        False,
    ),
    ("NLPL", "No Limit Public License", "https://spdx.org/licenses/NLPL.html", False),
    (
        "Nokia",
        "Nokia Open Source License",
        "https://spdx.org/licenses/Nokia.html",
        True,
    ),
    (
        "NCGL-UK-2.0",
        "Non-Commercial Government Licence",
        "https://spdx.org/licenses/NCGL-UK-2.0.html",
        False,
    ),
    (
        "NPOSL-3.0",
        "Non-Profit Open Software License 3.0",
        "https://spdx.org/licenses/NPOSL-3.0.html",
        True,
    ),
    (
        "NLOD-1.0",
        "Norwegian Licence for Open Government Data (NLOD) 1.0",
        "https://spdx.org/licenses/NLOD-1.0.html",
        False,
    ),
    (
        "NLOD-2.0",
        "Norwegian Licence for Open Government Data (NLOD) 2.0",
        "https://spdx.org/licenses/NLOD-2.0.html",
        False,
    ),
    ("Noweb", "Noweb License", "https://spdx.org/licenses/Noweb.html", False),
    ("NRL", "NRL License", "https://spdx.org/licenses/NRL.html", False),
    ("NTP", "NTP License", "https://spdx.org/licenses/NTP.html", True),
    ("NTP-0", "NTP No Attribution", "https://spdx.org/licenses/NTP-0.html", False),
    (
        "OCLC-2.0",
        "OCLC Research Public License 2.0",
        "https://spdx.org/licenses/OCLC-2.0.html",
        True,
    ),
    (
        "OGC-1.0",
        "OGC Software License, Version 1.0",
        "https://spdx.org/licenses/OGC-1.0.html",
        False,
    ),
    (
        "OCCT-PL",
        "Open CASCADE Technology Public License",
        "https://spdx.org/licenses/OCCT-PL.html",
        False,
    ),
    (
        "ODC-By-1.0",
        "Open Data Commons Attribution License v1.0",
        "https://spdx.org/licenses/ODC-By-1.0.html",
        False,
    ),
    (
        "ODbL-1.0",
        "Open Data Commons Open Database License v1.0",
        "https://spdx.org/licenses/ODbL-1.0.html",
        True,
    ),
    (
        "PDDL-1.0",
        "Open Data Commons Public Domain Dedication & License 1.0",
        "https://spdx.org/licenses/PDDL-1.0.html",
        False,
    ),
    (
        "OGL-Canada-2.0",
        "Open Government Licence - Canada",
        "https://spdx.org/licenses/OGL-Canada-2.0.html",
        False,
    ),
    (
        "OGL-UK-1.0",
        "Open Government Licence v1.0",
        "https://spdx.org/licenses/OGL-UK-1.0.html",
        False,
    ),
    (
        "OGL-UK-2.0",
        "Open Government Licence v2.0",
        "https://spdx.org/licenses/OGL-UK-2.0.html",
        False,
    ),
    (
        "OGL-UK-3.0",
        "Open Government Licence v3.0",
        "https://spdx.org/licenses/OGL-UK-3.0.html",
        False,
    ),
    (
        "OGTSL",
        "Open Group Test Suite License",
        "https://spdx.org/licenses/OGTSL.html",
        True,
    ),
    (
        "OLDAP-2.2.2",
        "Open LDAP Public License 2.2.2",
        "https://spdx.org/licenses/OLDAP-2.2.2.html",
        False,
    ),
    (
        "OLDAP-1.1",
        "Open LDAP Public License v1.1",
        "https://spdx.org/licenses/OLDAP-1.1.html",
        False,
    ),
    (
        "OLDAP-1.2",
        "Open LDAP Public License v1.2",
        "https://spdx.org/licenses/OLDAP-1.2.html",
        False,
    ),
    (
        "OLDAP-1.3",
        "Open LDAP Public License v1.3",
        "https://spdx.org/licenses/OLDAP-1.3.html",
        False,
    ),
    (
        "OLDAP-1.4",
        "Open LDAP Public License v1.4",
        "https://spdx.org/licenses/OLDAP-1.4.html",
        False,
    ),
    (
        "OLDAP-2.0",
        "Open LDAP Public License v2.0 (or possibly 2.0A and 2.0B)",
        "https://spdx.org/licenses/OLDAP-2.0.html",
        False,
    ),
    (
        "OLDAP-2.0.1",
        "Open LDAP Public License v2.0.1",
        "https://spdx.org/licenses/OLDAP-2.0.1.html",
        False,
    ),
    (
        "OLDAP-2.1",
        "Open LDAP Public License v2.1",
        "https://spdx.org/licenses/OLDAP-2.1.html",
        False,
    ),
    (
        "OLDAP-2.2",
        "Open LDAP Public License v2.2",
        "https://spdx.org/licenses/OLDAP-2.2.html",
        False,
    ),
    (
        "OLDAP-2.2.1",
        "Open LDAP Public License v2.2.1",
        "https://spdx.org/licenses/OLDAP-2.2.1.html",
        False,
    ),
    (
        "OLDAP-2.3",
        "Open LDAP Public License v2.3",
        "https://spdx.org/licenses/OLDAP-2.3.html",
        True,
    ),
    (
        "OLDAP-2.4",
        "Open LDAP Public License v2.4",
        "https://spdx.org/licenses/OLDAP-2.4.html",
        False,
    ),
    (
        "OLDAP-2.5",
        "Open LDAP Public License v2.5",
        "https://spdx.org/licenses/OLDAP-2.5.html",
        False,
    ),
    (
        "OLDAP-2.6",
        "Open LDAP Public License v2.6",
        "https://spdx.org/licenses/OLDAP-2.6.html",
        False,
    ),
    (
        "OLDAP-2.7",
        "Open LDAP Public License v2.7",
        "https://spdx.org/licenses/OLDAP-2.7.html",
        True,
    ),
    (
        "OLDAP-2.8",
        "Open LDAP Public License v2.8",
        "https://spdx.org/licenses/OLDAP-2.8.html",
        True,
    ),
    ("OML", "Open Market License", "https://spdx.org/licenses/OML.html", False),
    (
        "OPL-1.0",
        "Open Public License v1.0",
        "https://spdx.org/licenses/OPL-1.0.html",
        False,
    ),
    (
        "OPUBL-1.0",
        "Open Publication License v1.0",
        "https://spdx.org/licenses/OPUBL-1.0.html",
        False,
    ),
    (
        "OSL-1.0",
        "Open Software License 1.0",
        "https://spdx.org/licenses/OSL-1.0.html",
        True,
    ),
    (
        "OSL-1.1",
        "Open Software License 1.1",
        "https://spdx.org/licenses/OSL-1.1.html",
        True,
    ),
    (
        "OSL-2.0",
        "Open Software License 2.0",
        "https://spdx.org/licenses/OSL-2.0.html",
        True,
    ),
    (
        "OSL-2.1",
        "Open Software License 2.1",
        "https://spdx.org/licenses/OSL-2.1.html",
        True,
    ),
    (
        "OSL-3.0",
        "Open Software License 3.0",
        "https://spdx.org/licenses/OSL-3.0.html",
        True,
    ),
    (
        "O-UDA-1.0",
        "Open Use of Data Agreement v1.0",
        "https://spdx.org/licenses/O-UDA-1.0.html",
        False,
    ),
    ("OpenSSL", "OpenSSL License", "https://spdx.org/licenses/OpenSSL.html", True),
    (
        "OSET-PL-2.1",
        "OSET Public License version 2.1",
        "https://spdx.org/licenses/OSET-PL-2.1.html",
        True,
    ),
    ("PHP-3.0", "PHP License v3.0", "https://spdx.org/licenses/PHP-3.0.html", True),
    ("PHP-3.01", "PHP License v3.01", "https://spdx.org/licenses/PHP-3.01.html", True),
    (
        "Plexus",
        "Plexus Classworlds License",
        "https://spdx.org/licenses/Plexus.html",
        False,
    ),
    (
        "libpng-2.0",
        "PNG Reference Library version 2",
        "https://spdx.org/licenses/libpng-2.0.html",
        False,
    ),
    (
        "PolyForm-Noncommercial-1.0.0",
        "PolyForm Noncommercial License 1.0.0",
        "https://spdx.org/licenses/PolyForm-Noncommercial-1.0.0.html",
        False,
    ),
    (
        "PolyForm-Small-Business-1.0.0",
        "PolyForm Small Business License 1.0.0",
        "https://spdx.org/licenses/PolyForm-Small-Business-1.0.0.html",
        False,
    ),
    (
        "PostgreSQL",
        "PostgreSQL License",
        "https://spdx.org/licenses/PostgreSQL.html",
        True,
    ),
    ("psfrag", "psfrag License", "https://spdx.org/licenses/psfrag.html", False),
    ("psutils", "psutils License", "https://spdx.org/licenses/psutils.html", False),
    (
        "Python-2.0",
        "Python License 2.0",
        "https://spdx.org/licenses/Python-2.0.html",
        True,
    ),
    (
        "Python-2.0.1",
        "Python License 2.0.1",
        "https://spdx.org/licenses/Python-2.0.1.html",
        False,
    ),
    (
        "PSF-2.0",
        "Python Software Foundation License 2.0",
        "https://spdx.org/licenses/PSF-2.0.html",
        False,
    ),
    ("QPL-1.0", "Q Public License 1.0", "https://spdx.org/licenses/QPL-1.0.html", True),
    ("Qhull", "Qhull License", "https://spdx.org/licenses/Qhull.html", False),
    ("Rdisc", "Rdisc License", "https://spdx.org/licenses/Rdisc.html", False),
    (
        "RPSL-1.0",
        "RealNetworks Public Source License v1.0",
        "https://spdx.org/licenses/RPSL-1.0.html",
        True,
    ),
    (
        "RPL-1.1",
        "Reciprocal Public License 1.1",
        "https://spdx.org/licenses/RPL-1.1.html",
        True,
    ),
    (
        "RPL-1.5",
        "Reciprocal Public License 1.5",
        "https://spdx.org/licenses/RPL-1.5.html",
        True,
    ),
    (
        "RHeCos-1.1",
        "Red Hat eCos Public License v1.1",
        "https://spdx.org/licenses/RHeCos-1.1.html",
        False,
    ),
    (
        "RSCPL",
        "Ricoh Source Code Public License",
        "https://spdx.org/licenses/RSCPL.html",
        True,
    ),
    (
        "RSA-MD",
        "RSA Message-Digest License",
        "https://spdx.org/licenses/RSA-MD.html",
        False,
    ),
    ("Ruby", "Ruby License", "https://spdx.org/licenses/Ruby.html", True),
    (
        "SAX-PD",
        "Sax Public Domain Notice",
        "https://spdx.org/licenses/SAX-PD.html",
        False,
    ),
    ("Saxpath", "Saxpath License", "https://spdx.org/licenses/Saxpath.html", False),
    (
        "SCEA",
        "SCEA Shared Source License",
        "https://spdx.org/licenses/SCEA.html",
        False,
    ),
    (
        "SchemeReport",
        "Scheme Language Report License",
        "https://spdx.org/licenses/SchemeReport.html",
        False,
    ),
    (
        "SWL",
        "Scheme Widget Library (SWL) Software License Agreement",
        "https://spdx.org/licenses/SWL.html",
        False,
    ),
    (
        "SMPPL",
        "Secure Messaging Protocol Public License",
        "https://spdx.org/licenses/SMPPL.html",
        False,
    ),
    ("Sendmail", "Sendmail License", "https://spdx.org/licenses/Sendmail.html", False),
    (
        "Sendmail-8.23",
        "Sendmail License 8.23",
        "https://spdx.org/licenses/Sendmail-8.23.html",
        False,
    ),
    (
        "SSPL-1.0",
        "Server Side Public License, v 1",
        "https://spdx.org/licenses/SSPL-1.0.html",
        False,
    ),
    (
        "SGI-B-1.0",
        "SGI Free Software License B v1.0",
        "https://spdx.org/licenses/SGI-B-1.0.html",
        False,
    ),
    (
        "SGI-B-1.1",
        "SGI Free Software License B v1.1",
        "https://spdx.org/licenses/SGI-B-1.1.html",
        False,
    ),
    (
        "SGI-B-2.0",
        "SGI Free Software License B v2.0",
        "https://spdx.org/licenses/SGI-B-2.0.html",
        True,
    ),
    (
        "OFL-1.0",
        "SIL Open Font License 1.0",
        "https://spdx.org/licenses/OFL-1.0.html",
        True,
    ),
    (
        "OFL-1.0-no-RFN",
        "SIL Open Font License 1.0 with no Reserved Font Name",
        "https://spdx.org/licenses/OFL-1.0-no-RFN.html",
        False,
    ),
    (
        "OFL-1.0-RFN",
        "SIL Open Font License 1.0 with Reserved Font Name",
        "https://spdx.org/licenses/OFL-1.0-RFN.html",
        False,
    ),
    (
        "OFL-1.1",
        "SIL Open Font License 1.1",
        "https://spdx.org/licenses/OFL-1.1.html",
        True,
    ),
    (
        "OFL-1.1-no-RFN",
        "SIL Open Font License 1.1 with no Reserved Font Name",
        "https://spdx.org/licenses/OFL-1.1-no-RFN.html",
        True,
    ),
    (
        "OFL-1.1-RFN",
        "SIL Open Font License 1.1 with Reserved Font Name",
        "https://spdx.org/licenses/OFL-1.1-RFN.html",
        True,
    ),
    (
        "SimPL-2.0",
        "Simple Public License 2.0",
        "https://spdx.org/licenses/SimPL-2.0.html",
        True,
    ),
    (
        "Sleepycat",
        "Sleepycat License",
        "https://spdx.org/licenses/Sleepycat.html",
        True,
    ),
    ("SNIA", "SNIA Public License 1.1", "https://spdx.org/licenses/SNIA.html", False),
    (
        "SHL-0.5",
        "Solderpad Hardware License v0.5",
        "https://spdx.org/licenses/SHL-0.5.html",
        False,
    ),
    (
        "SHL-0.51",
        "Solderpad Hardware License, Version 0.51",
        "https://spdx.org/licenses/SHL-0.51.html",
        False,
    ),
    (
        "Spencer-86",
        "Spencer License 86",
        "https://spdx.org/licenses/Spencer-86.html",
        False,
    ),
    (
        "Spencer-94",
        "Spencer License 94",
        "https://spdx.org/licenses/Spencer-94.html",
        False,
    ),
    (
        "Spencer-99",
        "Spencer License 99",
        "https://spdx.org/licenses/Spencer-99.html",
        False,
    ),
    ("blessing", "SQLite Blessing", "https://spdx.org/licenses/blessing.html", False),
    (
        "SSH-OpenSSH",
        "SSH OpenSSH license",
        "https://spdx.org/licenses/SSH-OpenSSH.html",
        False,
    ),
    (
        "SSH-short",
        "SSH short notice",
        "https://spdx.org/licenses/SSH-short.html",
        False,
    ),
    (
        "SMLNJ",
        "Standard ML of New Jersey License",
        "https://spdx.org/licenses/SMLNJ.html",
        True,
    ),
    (
        "SugarCRM-1.1.3",
        "SugarCRM Public License v1.1.3",
        "https://spdx.org/licenses/SugarCRM-1.1.3.html",
        False,
    ),
    (
        "SISSL",
        "Sun Industry Standards Source License v1.1",
        "https://spdx.org/licenses/SISSL.html",
        True,
    ),
    (
        "SISSL-1.2",
        "Sun Industry Standards Source License v1.2",
        "https://spdx.org/licenses/SISSL-1.2.html",
        False,
    ),
    (
        "SPL-1.0",
        "Sun Public License v1.0",
        "https://spdx.org/licenses/SPL-1.0.html",
        True,
    ),
    (
        "Watcom-1.0",
        "Sybase Open Watcom Public License 1.0",
        "https://spdx.org/licenses/Watcom-1.0.html",
        True,
    ),
    ("Symlinks", "Symlinks License", "https://spdx.org/licenses/Symlinks.html", False),
    (
        "OGDL-Taiwan-1.0",
        "Taiwan Open Government Data License, version 1.0",
        "https://spdx.org/licenses/OGDL-Taiwan-1.0.html",
        False,
    ),
    (
        "TAPR-OHL-1.0",
        "TAPR Open Hardware License v1.0",
        "https://spdx.org/licenses/TAPR-OHL-1.0.html",
        False,
    ),
    ("TCL", "TCL/TK License", "https://spdx.org/licenses/TCL.html", False),
    (
        "TCP-wrappers",
        "TCP Wrappers License",
        "https://spdx.org/licenses/TCP-wrappers.html",
        False,
    ),
    (
        "TU-Berlin-1.0",
        "Technische Universitaet Berlin License 1.0",
        "https://spdx.org/licenses/TU-Berlin-1.0.html",
        False,
    ),
    (
        "TU-Berlin-2.0",
        "Technische Universitaet Berlin License 2.0",
        "https://spdx.org/licenses/TU-Berlin-2.0.html",
        False,
    ),
    ("TTWL", "Text-Tabs+Wrap License", "https://spdx.org/licenses/TTWL.html", False),
    ("MirOS", "The MirOS Licence", "https://spdx.org/licenses/MirOS.html", True),
    (
        "Parity-6.0.0",
        "The Parity Public License 6.0.0",
        "https://spdx.org/licenses/Parity-6.0.0.html",
        False,
    ),
    (
        "Parity-7.0.0",
        "The Parity Public License 7.0.0",
        "https://spdx.org/licenses/Parity-7.0.0.html",
        False,
    ),
    ("Unlicense", "The Unlicense", "https://spdx.org/licenses/Unlicense.html", True),
    ("TPDL", "Time::ParseDate License", "https://spdx.org/licenses/TPDL.html", False),
    (
        "TMate",
        "TMate Open Source License",
        "https://spdx.org/licenses/TMate.html",
        False,
    ),
    (
        "TORQUE-1.1",
        "TORQUE v2.5+ Software License v1.1",
        "https://spdx.org/licenses/TORQUE-1.1.html",
        False,
    ),
    (
        "TOSL",
        "Trusster Open Source License",
        "https://spdx.org/licenses/TOSL.html",
        False,
    ),
    (
        "Unicode-DFS-2015",
        "Unicode License Agreement - Data Files and Software (2015)",
        "https://spdx.org/licenses/Unicode-DFS-2015.html",
        False,
    ),
    (
        "Unicode-DFS-2016",
        "Unicode License Agreement - Data Files and Software (2016)",
        "https://spdx.org/licenses/Unicode-DFS-2016.html",
        True,
    ),
    (
        "Unicode-TOU",
        "Unicode Terms of Use",
        "https://spdx.org/licenses/Unicode-TOU.html",
        False,
    ),
    (
        "UPL-1.0",
        "Universal Permissive License v1.0",
        "https://spdx.org/licenses/UPL-1.0.html",
        True,
    ),
    (
        "NCSA",
        "University of Illinois/NCSA Open Source License",
        "https://spdx.org/licenses/NCSA.html",
        True,
    ),
    (
        "UCL-1.0",
        "Upstream Compatibility License v1.0",
        "https://spdx.org/licenses/UCL-1.0.html",
        True,
    ),
    ("Vim", "Vim License", "https://spdx.org/licenses/Vim.html", True),
    (
        "VOSTROM",
        "VOSTROM Public License for Open Source",
        "https://spdx.org/licenses/VOSTROM.html",
        False,
    ),
    (
        "VSL-1.0",
        "Vovida Software License v1.0",
        "https://spdx.org/licenses/VSL-1.0.html",
        True,
    ),
    (
        "W3C-20150513",
        "W3C Software Notice and Document License (2015-05-13)",
        "https://spdx.org/licenses/W3C-20150513.html",
        False,
    ),
    (
        "W3C-19980720",
        "W3C Software Notice and License (1998-07-20)",
        "https://spdx.org/licenses/W3C-19980720.html",
        False,
    ),
    (
        "W3C",
        "W3C Software Notice and License (2002-12-31)",
        "https://spdx.org/licenses/W3C.html",
        True,
    ),
    ("Wsuipa", "Wsuipa License", "https://spdx.org/licenses/Wsuipa.html", False),
    ("Xnet", "X.Net License", "https://spdx.org/licenses/Xnet.html", True),
    ("X11", "X11 License", "https://spdx.org/licenses/X11.html", True),
    (
        "X11-distribute-modifications-variant",
        "X11 License Distribution Modification Variant",
        "https://spdx.org/licenses/X11-distribute-modifications-variant.html",
        False,
    ),
    ("Xerox", "Xerox License", "https://spdx.org/licenses/Xerox.html", False),
    (
        "XFree86-1.1",
        "XFree86 License 1.1",
        "https://spdx.org/licenses/XFree86-1.1.html",
        True,
    ),
    ("xinetd", "xinetd License", "https://spdx.org/licenses/xinetd.html", True),
    ("xpp", "XPP License", "https://spdx.org/licenses/xpp.html", False),
    ("XSkat", "XSkat License", "https://spdx.org/licenses/XSkat.html", False),
    (
        "YPL-1.0",
        "Yahoo! Public License v1.0",
        "https://spdx.org/licenses/YPL-1.0.html",
        False,
    ),
    (
        "YPL-1.1",
        "Yahoo! Public License v1.1",
        "https://spdx.org/licenses/YPL-1.1.html",
        True,
    ),
    ("Zed", "Zed License", "https://spdx.org/licenses/Zed.html", False),
    ("Zend-2.0", "Zend License v2.0", "https://spdx.org/licenses/Zend-2.0.html", True),
    (
        "Zimbra-1.3",
        "Zimbra Public License v1.3",
        "https://spdx.org/licenses/Zimbra-1.3.html",
        True,
    ),
    (
        "Zimbra-1.4",
        "Zimbra Public License v1.4",
        "https://spdx.org/licenses/Zimbra-1.4.html",
        False,
    ),
    ("Zlib", "zlib License", "https://spdx.org/licenses/Zlib.html", True),
    (
        "zlib-acknowledgement",
        "zlib/libpng License with Acknowledgement",
        "https://spdx.org/licenses/zlib-acknowledgement.html",
        False,
    ),
    (
        "ZPL-1.1",
        "Zope Public License 1.1",
        "https://spdx.org/licenses/ZPL-1.1.html",
        False,
    ),
    (
        "ZPL-2.0",
        "Zope Public License 2.0",
        "https://spdx.org/licenses/ZPL-2.0.html",
        True,
    ),
    (
        "ZPL-2.1",
        "Zope Public License 2.1",
        "https://spdx.org/licenses/ZPL-2.1.html",
        True,
    ),
)



# Copyright © Michal Čihař <michal@weblate.org>
#
# SPDX-License-Identifier: GPL-3.0-or-later

from weblate.auth.models import create_groups, setup_project_groups
from weblate.trans.models import Project
from weblate.utils.management.base import BaseCommand


class Command(BaseCommand):
    help = "setups default user groups"

    def add_arguments(self, parser):
        parser.add_argument(
            "--no-privs-update",
            action="store_false",
            dest="update",
            default=True,
            help="Prevents updates of privileges of existing groups",
        )
        parser.add_argument(
            "--no-projects-update",
            action="store_false",
            dest="projects",
            default=True,
            help="Prevents updates of groups for existing projects",
        )

    def handle(self, *args, **options):
        """Create or update default set of groups.

        It also optionally updates them and moves users around to default group.
        """
        create_groups(options["update"])
        if options["projects"]:
            for project in Project.objects.iterator():
                setup_project_groups(Project, project)



# Copyright © Michal Čihař <michal@weblate.org>
#
# SPDX-License-Identifier: GPL-3.0-or-later

# Generated by Django 4.1.3 on 2022-12-12 10:40

from django.db import migrations


def rename_index(apps, schema_editor):
    if schema_editor.connection.vendor != "postgresql":
        return
    schema_editor.execute(
        "ALTER INDEX memory_source_trgm_tmp RENAME TO memory_source_trgm"
    )


class Migration(migrations.Migration):

    dependencies = [
        ("memory", "0013_reindex"),
    ]

    operations = [
        migrations.RunPython(
            rename_index, migrations.RunPython.noop, elidable=False, atomic=False
        )
    ]



# Copyright © Michal Čihař <michal@weblate.org>
#
# SPDX-License-Identifier: GPL-3.0-or-later

# Generated by Django 3.2 on 2022-02-23 07:33

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("wladmin", "0004_alter_backupservice_repository"),
    ]

    operations = [
        migrations.AlterField(
            model_name="backuplog",
            name="event",
            field=models.CharField(
                choices=[
                    ("backup", "Backup performed"),
                    ("error", "Backup failed"),
                    ("prune", "Deleted the oldest backups"),
                    ("cleanup", "Cleaned up backup storage"),
                    ("init", "Repository initialization"),
                ],
                db_index=True,
                max_length=100,
            ),
        ),
    ]



# Copyright © Michal Čihař <michal@weblate.org>
#
# SPDX-License-Identifier: GPL-3.0-or-later

from django.apps import AppConfig
from django.db.models.signals import post_migrate


class AuthConfig(AppConfig):
    name = "weblate.auth"
    label = "weblate_auth"
    verbose_name = "Authentication"

    def ready(self):
        from weblate.auth.models import sync_create_groups

        post_migrate.connect(sync_create_groups, sender=self)



# Copyright © Michal Čihař <michal@weblate.org>
#
# SPDX-License-Identifier: GPL-3.0-or-later

# Generated by Django 4.1b1 on 2022-07-18 12:46

from django.db import migrations


def rename_java_language(apps, schema_editor):
    Component = apps.get_model("trans", "Component")

    Component.objects.using(schema_editor.connection.alias).filter(
        language_code_style="java"
    ).update(language_code_style="bcp_legacy")


class Migration(migrations.Migration):

    dependencies = [
        ("trans", "0152_alter_change_action"),
    ]

    operations = [migrations.RunPython(rename_java_language, elidable=True)]



# Copyright © Michal Čihař <michal@weblate.org>
#
# SPDX-License-Identifier: GPL-3.0-or-later

import os
from collections import namedtuple

from weblate.vcs.base import RepositoryException
from weblate.vcs.git import GitRepository

# This has to stay here for compatibility reasons - it is stored pickled in
# the cache and moving it around breaks ugprades.
Release = namedtuple("Release", ["version", "timestamp"])


def get_root_dir():
    """Return Weblate root dir."""
    curdir = os.path.dirname(os.path.abspath(__file__))
    return os.path.abspath(os.path.join(curdir, "..", ".."))


# Weblate version
VERSION = "4.15.2"

# Version string without suffix
VERSION_BASE = VERSION.replace("-dev", "")

# User-Agent string to use
USER_AGENT = f"Weblate/{VERSION}"

# Git tag name for this release
TAG_NAME = f"weblate-{VERSION_BASE}"

# Grab some information from git
try:
    # Describe current checkout
    GIT_REPO = GitRepository(get_root_dir(), local=True)
    GIT_VERSION = GIT_REPO.describe()
    GIT_REVISION = GIT_REPO.last_revision
    del GIT_REPO
except (RepositoryException, OSError):
    # Special case for Docker bleeding builds
    if "WEBLATE_DOCKER_GIT_REVISION" in os.environ:
        GIT_REVISION = os.environ["WEBLATE_DOCKER_GIT_REVISION"]
        GIT_VERSION = f"{VERSION_BASE}-{GIT_REVISION[:10]}"
    else:
        # Import failed or git has troubles reading
        # repo (for example swallow clone)
        GIT_VERSION = VERSION
        GIT_REVISION = None

if GIT_REVISION:
    GIT_LINK = f"https://github.com/WeblateOrg/weblate/commits/{GIT_REVISION}"
elif "-dev" not in VERSION:
    GIT_LINK = f"https://github.com/WeblateOrg/weblate/releases/tag/weblate-{VERSION}"
else:
    GIT_LINK = None



# Copyright © Michal Čihař <michal@weblate.org>
#
# SPDX-License-Identifier: GPL-3.0-or-later

"""Tests for Unicode sorting."""


from unittest import SkipTest

from django.test import TestCase

import weblate.trans.util


class SortTest(TestCase):
    def test_sort(self):
        if not weblate.trans.util.LOCALE_SETUP:
            raise SkipTest("Could not set up locales")
        result = weblate.trans.util.sort_choices(
            ((2, "zkouška"), (3, "zkouzka"), (1, "zkouaka"))
        )
        self.assertEqual([1, 2, 3], [x[0] for x in result])



# Copyright © Michal Čihař <michal@weblate.org>
#
# SPDX-License-Identifier: GPL-3.0-or-later

"""Captcha tests."""

from unittest import TestCase

from weblate.accounts.captcha import MathCaptcha


class CaptchaTest(TestCase):
    def test_object(self):
        captcha = MathCaptcha("1 * 2")
        self.assertFalse(captcha.validate(1))
        self.assertTrue(captcha.validate(2))
        restored = MathCaptcha.unserialize(captcha.serialize())
        self.assertEqual(captcha.question, restored.question)
        self.assertTrue(restored.validate(2))

    def test_generate(self):
        """Test generating of captcha for every operator."""
        captcha = MathCaptcha()
        for operator in MathCaptcha.operators:
            captcha.operators = (operator,)
            self.assertIn(operator, captcha.generate_question())



# Copyright © Michal Čihař <michal@weblate.org>
#
# SPDX-License-Identifier: GPL-3.0-or-later

from weblate.trans.management.commands import WeblateLangCommand
from weblate.trans.tasks import perform_load


class Command(WeblateLangCommand):
    help = "(re)loads translations from disk"

    def add_arguments(self, parser):
        super().add_arguments(parser)
        parser.add_argument(
            "--force",
            action="store_true",
            default=False,
            help="Force rereading files even when they should be up to date",
        )
        parser.add_argument(
            "--foreground",
            action="store_true",
            default=False,
            help="Perform load in foreground (by default background task is used)",
        )

    def handle(self, *args, **options):
        langs = None
        if options["lang"] is not None:
            langs = options["lang"].split(",")
        if options["foreground"]:
            loader = perform_load
        else:
            loader = perform_load.delay
        for component in self.get_components(**options):
            loader(component.pk, force=options["force"], langs=langs)



# Copyright © Michal Čihař <michal@weblate.org>
#
# SPDX-License-Identifier: GPL-3.0-or-later

from weblate.billing.models import Billing
from weblate.billing.tasks import billing_notify
from weblate.utils.management.base import BaseCommand


class Command(BaseCommand):
    """Command for billing check."""

    help = "checks billing limits"

    def add_arguments(self, parser):
        parser.add_argument("--valid", action="store_true", help="list valid ones")
        parser.add_argument(
            "--notify", action="store_true", help="send email notifications"
        )

    def handle(self, *args, **options):
        if options["notify"]:
            billing_notify()
            return
        Billing.objects.check_limits()
        if options["valid"]:
            for bill in Billing.objects.get_valid():
                self.stdout.write(f" * {bill}")
            return
        limit = Billing.objects.get_out_of_limits()
        due = Billing.objects.get_unpaid()

        if limit:
            self.stdout.write("Following billings are over limit:")
            for bill in limit:
                self.stdout.write(f" * {bill}")

        if due:
            self.stdout.write("Following billings are past due date:")
            for bill in due:
                self.stdout.write(f" * {bill}")



# Copyright © Michal Čihař <michal@weblate.org>
#
# SPDX-License-Identifier: GPL-3.0-or-later

from django.test import SimpleTestCase

from weblate.utils.hash import (
    calculate_checksum,
    calculate_hash,
    checksum_to_hash,
    hash_to_checksum,
)


class HashTest(SimpleTestCase):
    def test_hash(self):
        """Ensure hash is not changing."""
        text = "Message"
        text_hash = calculate_hash(text)
        self.assertEqual(text_hash, 8445691827737211251)
        self.assertEqual(text_hash, calculate_hash(text))

    def test_hash_context(self):
        """Ensure hash works with context."""
        text = "Message"
        context = "Context"
        text_hash = calculate_hash(context, text)
        self.assertEqual(text_hash, -1602104568316855346)
        self.assertEqual(text_hash, calculate_hash(context, text))

    def test_hash_unicode(self):
        """Ensure hash works for unicode."""
        text = "Příšerně žluťoučký kůň úpěl ďábelské ódy"
        text_hash = calculate_hash(text)
        self.assertEqual(text_hash, -4296353750398394478)
        self.assertEqual(text_hash, calculate_hash(text))

    def test_checksum(self):
        """Hash to checksum conversion."""
        text_hash = calculate_hash("Message")
        checksum = hash_to_checksum(text_hash)
        self.assertEqual(checksum, "f5351ff85ab23173")
        self.assertEqual(text_hash, checksum_to_hash(checksum))

    def test_calculate_checksum(self):
        self.assertEqual(calculate_checksum("Message"), "f5351ff85ab23173")



# Copyright © Michal Čihař <michal@weblate.org>
#
# SPDX-License-Identifier: GPL-3.0-or-later

from django.apps import AppConfig
from django.utils.translation import gettext_lazy as _


class GlossaryConfig(AppConfig):
    name = "weblate.glossary"
    label = "glossary"
    verbose_name = _("Glossaries")



# Copyright © Michal Čihař <michal@weblate.org>
#
# SPDX-License-Identifier: GPL-3.0-or-later

from django.db import transaction

from weblate.accounts.tasks import cleanup_social_auth
from weblate.screenshots.tasks import cleanup_screenshot_files
from weblate.trans.models import Project
from weblate.trans.tasks import (
    cleanup_old_comments,
    cleanup_old_suggestions,
    cleanup_project,
    cleanup_stale_repos,
    cleanup_suggestions,
)
from weblate.utils.management.base import BaseCommand


class Command(BaseCommand):
    help = "clenups orphaned checks and suggestions"

    def handle(self, *args, **options):
        """Perform cleanup of Weblate database."""
        cleanup_screenshot_files()
        with transaction.atomic():
            cleanup_social_auth()
        for project in Project.objects.values_list("id", flat=True):
            cleanup_project(project)
        cleanup_suggestions()
        cleanup_stale_repos()
        cleanup_old_suggestions()
        cleanup_old_comments()



# Copyright © Michal Čihař <michal@weblate.org>
#
# SPDX-License-Identifier: GPL-3.0-or-later

from django.utils.translation import gettext_lazy

COLOR_CHOICES = (
    # Translators: Name of a color
    ("navy", gettext_lazy("Navy")),
    # Translators: Name of a color
    ("blue", gettext_lazy("Blue")),
    # Translators: Name of a color
    ("aqua", gettext_lazy("Aqua")),
    # Translators: Name of a color
    ("teal", gettext_lazy("Teal")),
    # Translators: Name of a color
    ("olive", gettext_lazy("Olive")),
    # Translators: Name of a color
    ("green", gettext_lazy("Green")),
    # Translators: Name of a color
    ("lime", gettext_lazy("Lime")),
    # Translators: Name of a color
    ("yellow", gettext_lazy("Yellow")),
    # Translators: Name of a color
    ("orange", gettext_lazy("Orange")),
    # Translators: Name of a color
    ("red", gettext_lazy("Red")),
    # Translators: Name of a color
    ("maroon", gettext_lazy("Maroon")),
    # Translators: Name of a color
    ("fuchsia", gettext_lazy("Fuchsia")),
    # Translators: Name of a color
    ("purple", gettext_lazy("Purple")),
    # Translators: Name of a color
    ("black", gettext_lazy("Black")),
    # Translators: Name of a color
    ("gray", gettext_lazy("Gray")),
    # Translators: Name of a color
    ("silver", gettext_lazy("Silver")),
)



# Copyright © Michal Čihař <michal@weblate.org>
#
# SPDX-License-Identifier: GPL-3.0-or-later

"""Tests for placeholder quality checks."""

from weblate.checks.flags import Flags
from weblate.checks.models import Check
from weblate.checks.placeholders import PlaceholderCheck, RegexCheck
from weblate.checks.tests.test_checks import CheckTestCase, MockUnit
from weblate.lang.models import Language, Plural
from weblate.trans.models import Component, Project, Translation, Unit
from weblate.trans.tests.test_views import FixtureTestCase


class PlaceholdersTest(CheckTestCase):
    check = PlaceholderCheck()

    def setUp(self):
        super().setUp()
        self.test_good_matching = ("string $URL$", "string $URL$", "placeholders:$URL$")
        self.test_good_none = ("string", "string", "placeholders:")
        self.test_good_ignore = ("$URL", "$OTHER", "")
        self.test_failure_1 = ("string $URL$", "string", "placeholders:$URL$")
        self.test_failure_2 = ("string $URL$", "string $URL", "placeholders:$URL$")
        self.test_failure_3 = (
            "string $URL$ $2$",
            "string $URL$",
            "placeholders:$URL$:$2$:",
        )
        self.test_highlight = ("placeholders:$URL$", "See $URL$", [(4, 9, "$URL$")])

    def do_test(self, expected, data, lang=None):
        # Skip using check_single as the Check does not use that
        return

    def test_description(self):
        unit = Unit(
            source="string $URL$",
            target="string",
            translation=Translation(
                component=Component(
                    project=Project(slug="p", name="p"),
                    source_language=Language(),
                    slug="c",
                    name="c",
                    pk=-1,
                ),
                language=Language(),
                plural=Plural(),
            ),
        )
        unit.__dict__["all_flags"] = Flags("placeholders:$URL$")
        check = Check(unit=unit)
        self.assertHTMLEqual(
            self.check.get_description(check),
            """
            Following format strings are missing:
            <span class="hlcheck" data-value="$URL$">$URL$</span>
            """,
        )

    def test_regexp(self):
        unit = Unit(
            source="string $URL$",
            target="string $FOO$",
            translation=Translation(
                component=Component(
                    project=Project(slug="p", name="p"),
                    source_language=Language(),
                    slug="c",
                    name="c",
                    pk=-1,
                ),
                language=Language(),
                plural=Plural(),
            ),
        )
        unit.__dict__["all_flags"] = Flags(r"""placeholders:r"(\$)([^$]*)(\$)" """)
        check = Check(unit=unit)
        self.assertHTMLEqual(
            self.check.get_description(check),
            """
            Following format strings are missing:
            <span class="hlcheck" data-value="$URL$">$URL$</span>
            <br />
            Following format strings are extra:
            <span class="hlcheck" data-value="$FOO$">$FOO$</span>
            """,
        )

    def test_whitespace(self):
        unit = Unit(
            source="string {URL} ",
            target="string {URL}",
            translation=Translation(
                component=Component(
                    project=Project(slug="p", name="p"),
                    source_language=Language(),
                    slug="c",
                    name="c",
                    pk=-1,
                ),
                language=Language(),
                plural=Plural(),
            ),
        )
        unit.__dict__["all_flags"] = Flags(r"""placeholders:r"\s?{\w+}\s?" """)
        check = Check(unit=unit)
        self.assertHTMLEqual(
            self.check.get_description(check),
            """
            Following format strings are missing:
            <span class="hlcheck" data-value=" {URL} ">
            <span class="hlspace"><span class="space-space">
            </span></span>
            {URL}
            <span class="hlspace"><span class="space-space">
            </span></span>
            </span>
            <br />
            Following format strings are extra:
            <span class="hlcheck" data-value=" {URL}">
            <span class="hlspace"><span class="space-space">
            </span></span>
            {URL}
            </span>
            """,
        )

    def test_case_insentivive(self):
        self.assertTrue(
            self.check.check_target(
                ["Hello %WORLD%"],
                ["Ahoj %world%"],
                MockUnit(
                    None,
                    "placeholders:%WORLD%",
                    self.default_lang,
                    "Hello %WORLD%",
                ),
            )
        )
        self.assertFalse(
            self.check.check_target(
                ["Hello %WORLD%"],
                ["Ahoj %world%"],
                MockUnit(
                    None,
                    "placeholders:%WORLD%,case-insensitive",
                    self.default_lang,
                    "Hello %WORLD%",
                ),
            )
        )


class PluralPlaceholdersTest(FixtureTestCase):
    def test_plural(self):
        check = PlaceholderCheck()
        lang = "cs"
        unit = MockUnit(
            None,
            'placeholders:r"%[0-9]"',
            lang,
            "1 apple",
        )
        unit.translation.language = Language.objects.get(code=lang)
        unit.translation.plural = unit.translation.language.plural
        self.assertFalse(
            check.check_target(
                ["1 apple", "%1 apples"],
                ["1 jablko", "%1 jablka", "%1 jablek"],
                unit,
            )
        )
        unit.check_cache = {}
        self.assertTrue(
            check.check_target(
                ["1 apple", "%1 apples"],
                ["1 jablko", "1 jablka", "%1 jablek"],
                unit,
            )
        )
        unit.check_cache = {}
        self.assertTrue(
            check.check_target(
                ["%1 apple", "%1 apples"],
                ["1 jablko", "1 jablka", "%1 jablek"],
                unit,
            )
        )


class RegexTest(CheckTestCase):
    check = RegexCheck()

    def setUp(self):
        super().setUp()
        self.test_good_matching = ("string URL", "string URL", "regex:URL")
        self.test_good_none = ("string", "string", "regex:")
        self.test_failure_1 = ("string URL", "string", "regex:URL")
        self.test_failure_2 = ("string URL", "string url", "regex:URL")
        self.test_failure_3 = ("string URL", "string URL", "regex:^URL$")
        self.test_highlight = ("regex:URL", "See URL", [(4, 7, "URL")])

    def do_test(self, expected, data, lang=None):
        # Skip using check_single as the Check does not use that
        return

    def test_description(self):
        unit = Unit(source="string URL", target="string")
        unit.__dict__["all_flags"] = Flags("regex:URL")
        check = Check(unit=unit)
        self.assertEqual(
            self.check.get_description(check),
            "Does not match regular expression <code>URL</code>.",
        )

    def test_check_highlight_groups(self):
        unit = MockUnit(
            None,
            r'regex:"((?:@:\(|\{)[^\)\}]+(?:\)|\}))"',
            self.default_lang,
            "@:(foo.bar.baz) | @:(hello.world) | {foo32}",
        )
        self.assertEqual(
            list(self.check.check_highlight(unit.source, unit)),
            [
                (0, 15, "@:(foo.bar.baz)"),
                (18, 33, "@:(hello.world)"),
                (36, 43, "{foo32}"),
            ],
        )



# Copyright © Michal Čihař <michal@weblate.org>
#
# SPDX-License-Identifier: GPL-3.0-or-later

"""Tests for data exports."""

from django.urls import reverse

from weblate.trans.tests.test_views import FixtureTestCase


class BasicViewTest(FixtureTestCase):
    def test_about(self):
        response = self.client.get(reverse("about"))
        self.assertContains(response, "translate-toolkit")

    def test_keys(self):
        response = self.client.get(reverse("keys"))
        self.assertContains(response, "SSH")

    def test_stats(self):
        response = self.client.get(reverse("stats"))
        self.assertContains(response, "Weblate statistics")

    def test_healthz(self):
        response = self.client.get(reverse("healthz"))
        self.assertContains(response, "ok")



# Copyright © Michal Čihař <michal@weblate.org>
#
# SPDX-License-Identifier: GPL-3.0-or-later

import re
from typing import Optional

from django.core.exceptions import ValidationError
from django.utils.translation import gettext as _
from pyparsing import ParseException

from weblate.checks.flags import Flags
from weblate.lang.models import Language
from weblate.trans.defines import LANGUAGE_CODE_LENGTH


def validate_filemask(val):
    """Validate that filemask contains *."""
    if "*" not in val:
        raise ValidationError(
            _("File mask does not contain * as a language placeholder!")
        )


def validate_autoaccept(val):
    """Validate correct value for autoaccept."""
    if val == 1:
        raise ValidationError(
            _(
                "A value of 1 is not allowed for autoaccept as "
                "it would permit users to vote on their own suggestions."
            )
        )


def validate_check_flags(val):
    """Validate check influencing flags."""
    try:
        flags = Flags(val)
    except (ParseException, re.error) as error:
        raise ValidationError(_("Failed to parse flags: %s") % error)
    flags.validate()


def validate_language_code(code: Optional[str], filename: str, required: bool = False):
    if not code:
        if not required:
            return None
        message = _(
            'The language code for "%(filename)s" is empty, please check the file mask.'
        ) % {"filename": filename}
        raise ValidationError({"filemask": message})

    if len(code) > LANGUAGE_CODE_LENGTH:
        message = _(
            'The language code "%(code)s" for "%(filename)s" is too long,'
            " please check the file mask."
        ) % {"code": code, "filename": filename}
        raise ValidationError({"filemask": message})

    return Language.objects.auto_get_or_create(code=code, create=False)



# Copyright © Michal Čihař <michal@weblate.org>
#
# SPDX-License-Identifier: GPL-3.0-or-later

from datetime import date

from weblate.metrics.models import Metric
from weblate.metrics.tasks import collect_metrics
from weblate.utils.management.base import BaseCommand
from weblate.utils.stats import GlobalStats


class Command(BaseCommand):
    help = "ensures that stats are present"

    def handle(self, *args, **options):
        GlobalStats().ensure_basic()
        if not Metric.objects.filter(
            date=date.today(), scope=Metric.SCOPE_GLOBAL
        ).exists():
            collect_metrics()



# Copyright © Michal Čihař <michal@weblate.org>
#
# SPDX-License-Identifier: GPL-3.0-or-later

from unittest import TestCase

from django.http.request import HttpRequest

from weblate.utils.request import get_ip_address, get_user_agent


class RequestTest(TestCase):
    def test_get_ip(self):
        request = HttpRequest()
        request.META["REMOTE_ADDR"] = "1.2.3.4"
        self.assertEqual(get_ip_address(request), "1.2.3.4")

    def test_agent(self):
        request = HttpRequest()
        request.META["HTTP_USER_AGENT"] = "agent"
        self.assertEqual(get_user_agent(request), "Other / Other / Other")

    def test_agent_long(self):
        request = HttpRequest()
        request.META["HTTP_USER_AGENT"] = "agent " * 200
        self.assertLess(len(get_user_agent(request)), 200)



# Copyright © Michal Čihař <michal@weblate.org>
#
# SPDX-License-Identifier: GPL-3.0-or-later

from weblate.lang.models import Language, Plural
from weblate.utils.management.base import BaseCommand


class Command(BaseCommand):
    help = "Move all content from one language to other"

    def add_arguments(self, parser):
        parser.add_argument("source", help="Source language code")
        parser.add_argument("target", help="Target language code")

    def handle(self, *args, **options):
        source = Language.objects.get(code=options["source"])
        target = Language.objects.get(code=options["target"])

        for translation in source.translation_set.iterator():
            other = translation.component.translation_set.filter(language=target)
            if other.exists():
                self.stderr.write(f"Already exists: {translation}")
                continue
            translation.language = target
            translation.save()
        source.announcement_set.update(language=target)

        for profile in source.profile_set.iterator():
            profile.languages.remove(source)
            profile.languages.add(target)

        for profile in source.secondary_profile_set.iterator():
            profile.secondary_languages.remove(source)
            profile.secondary_languages.add(target)

        source.change_set.update(language=target)

        source.component_set.update(source_language=target)
        for group in source.group_set.iterator():
            group.languages.remove(source)
            group.languages.add(target)

        for plural in source.plural_set.iterator():
            try:
                new_plural = target.plural_set.filter(formula=plural.formula).first()
                plural.translation_set.update(plural=new_plural)
            except Plural.DoesNotExist:
                plural.language = target
                plural.save()



# Copyright © Michal Čihař <michal@weblate.org>
#
# SPDX-License-Identifier: GPL-3.0-or-later

from django.contrib import admin

from weblate.fonts.models import FontOverride
from weblate.wladmin.models import WeblateModelAdmin


class FontAdmin(WeblateModelAdmin):
    list_display = ["family", "style", "project", "user"]
    search_fields = ["family", "style"]
    list_filter = [("project", admin.RelatedOnlyFieldListFilter)]
    ordering = ["family", "style"]


class InlineFontOverrideAdmin(admin.TabularInline):
    model = FontOverride
    extra = 0


class FontGroupAdmin(WeblateModelAdmin):
    list_display = ["name", "font", "project"]
    search_fields = ["name", "font__family"]
    list_filter = [("project", admin.RelatedOnlyFieldListFilter)]
    ordering = ["name"]
    inlines = [InlineFontOverrideAdmin]



# Copyright © Michal Čihař <michal@weblate.org>
#
# SPDX-License-Identifier: GPL-3.0-or-later

from django.utils.translation import gettext_lazy as _

from weblate.addons.base import StoreBaseAddon
from weblate.addons.forms import YAMLCustomizeForm

BREAKS = {"dos": "\r\n", "mac": "\r", "unix": "\n"}


class YAMLCustomizeAddon(StoreBaseAddon):
    name = "weblate.yaml.customize"
    verbose = _("Customize YAML output")
    description = _(
        "Allows adjusting YAML output behavior, for example line-length or newlines."
    )
    settings_form = YAMLCustomizeForm
    compat = {"file_format": {"yaml", "ruby-yaml"}}

    def store_post_load(self, translation, store):
        config = self.instance.configuration

        args = store.store.dump_args

        args["indent"] = int(config.get("indent", 2))
        args["width"] = int(config.get("width", 80))
        args["line_break"] = BREAKS[config.get("line_break", "unix")]



# Copyright © Michal Čihař <michal@weblate.org>
#
# SPDX-License-Identifier: GPL-3.0-or-later

from django import template

from weblate.auth.models import User
from weblate.lang.models import Language
from weblate.metrics.models import Metric
from weblate.metrics.wrapper import MetricsWrapper
from weblate.trans.models import Component, ComponentList, Project, Translation
from weblate.utils.stats import ProjectLanguage

register = template.Library()


@register.filter
def metrics(obj):
    if obj is None:
        return MetricsWrapper(obj, Metric.SCOPE_GLOBAL, 0)
    if isinstance(obj, Translation):
        return MetricsWrapper(obj, Metric.SCOPE_TRANSLATION, obj.pk)
    if isinstance(obj, Component):
        return MetricsWrapper(obj, Metric.SCOPE_COMPONENT, obj.pk)
    if isinstance(obj, Project):
        return MetricsWrapper(obj, Metric.SCOPE_PROJECT, obj.pk)
    if isinstance(obj, ComponentList):
        return MetricsWrapper(obj, Metric.SCOPE_COMPONENT_LIST, obj.pk)
    if isinstance(obj, ProjectLanguage):
        return MetricsWrapper(
            obj, Metric.SCOPE_PROJECT_LANGUAGE, obj.project.id, obj.language.id
        )
    if isinstance(obj, Language):
        return MetricsWrapper(obj, Metric.SCOPE_LANGUAGE, obj.id)
    if isinstance(obj, User):
        return MetricsWrapper(obj, Metric.SCOPE_USER, obj.id)
    raise ValueError(f"Unsupported type for metrics: {obj!r}")



# Copyright © Michal Čihař <michal@weblate.org>
#
# SPDX-License-Identifier: GPL-3.0-or-later

from django.urls import reverse

from weblate.machinery.base import MachineTranslation, get_machinery_language
from weblate.memory.models import Memory


class WeblateMemory(MachineTranslation):
    """Translation service using strings already translated in Weblate."""

    name = "Weblate Translation Memory"
    rank_boost = 2
    cache_translations = False
    same_languages = True
    accounting_key = "internal"
    do_cleanup = False

    def convert_language(self, language):
        """No conversion of language object."""
        return get_machinery_language(language)

    def is_supported(self, source, language):
        """Any language is supported."""
        return True

    def is_rate_limited(self):
        """This service has no rate limiting."""
        return False

    def download_translations(
        self,
        source,
        language,
        text: str,
        unit,
        user,
        search: bool,
        threshold: int = 75,
    ):
        """Download list of possible translations from a service."""
        for result in Memory.objects.lookup(
            source,
            language,
            text,
            user,
            unit.translation.component.project,
            unit.translation.component.project.use_shared_tm,
        ).iterator():
            quality = self.comparer.similarity(text, result.source)
            if quality < 10 or (quality < threshold and not search):
                continue
            yield {
                "text": result.target,
                "quality": quality,
                "service": self.name,
                "origin": result.get_origin_display(),
                "source": result.source,
                "show_quality": True,
                "delete_url": reverse("api:memory-detail", kwargs={"pk": result.id})
                if user is not None and user.has_perm("memory.delete", result)
                else None,
            }



# Copyright © Michal Čihař <michal@weblate.org>
# Copyright © WofWca <wofwca@protonmail.com>
#
# SPDX-License-Identifier: GPL-3.0-or-later

from django.contrib.auth.decorators import login_required
from django.http import HttpResponse, JsonResponse
from django.utils.html import conditional_escape, format_html, format_html_join
from django.views.decorators.http import require_POST

from weblate.lang.models import Language
from weblate.trans.forms import ReportsForm
from weblate.trans.models.change import Change
from weblate.trans.util import redirect_param
from weblate.utils.views import get_component, get_project, show_form_errors

# Header, two longer fields for name and email, shorter fields for numbers
RST_HEADING = " ".join(["=" * 40] * 2 + ["=" * 24] * 20)

HTML_HEADING = "<table>\n<tr>{0}</tr>"


def format_plaintext(format_string, *args, **kwargs):
    """Same as `format_html` in syntax, but performs no escaping."""
    return format_string.format(*args, **kwargs)


def format_plaintext_join(sep, format_string, args_generator):
    """Same as `format_html_join` in syntax, but performs no escaping."""
    return sep.join(format_plaintext(format_string, *args) for args in args_generator)


def generate_credits(user, start_date, end_date, **kwargs):
    """Generate credits data for given component."""
    result = []

    base = Change.objects.content()
    if user:
        base = base.filter(author=user)

    for language in Language.objects.filter(**kwargs).distinct().iterator():
        authors = base.filter(language=language, **kwargs).authors_list(
            (start_date, end_date)
        )
        if not authors:
            continue
        result.append({language.name: sorted(authors, key=lambda item: item[2])})

    return result


@login_required
@require_POST
def get_credits(request, project=None, component=None):
    """View for credits."""
    if project is None:
        obj = None
        kwargs = {"translation__isnull": False}
    elif component is None:
        obj = get_project(request, project)
        kwargs = {"translation__component__project": obj}
    else:
        obj = get_component(request, project, component)
        kwargs = {"translation__component": obj}

    form = ReportsForm(request.POST)

    if not form.is_valid():
        show_form_errors(request, form)
        return redirect_param(obj or "home", "#reports")

    data = generate_credits(
        None if request.user.has_perm("reports.view", obj) else request.user,
        form.cleaned_data["start_date"],
        form.cleaned_data["end_date"],
        **kwargs,
    )

    if form.cleaned_data["style"] == "json":
        return JsonResponse(data=data, safe=False)

    if form.cleaned_data["style"] == "html":
        wrap_format = "<table><tbody>{}</tbody></table>"
        language_format = """
        <tr>
            <th>{language}</th>
            <td><ul>{translators}</ul></td>
        </tr>
        """
        translator_format = '<li><a href="mailto:{0}">{1}</a> ({2})</li>'
        mime = "text/html"
        format_html_or_plain = format_html
        format_html_or_plain_join = format_html_join
    else:
        wrap_format = "{}"
        language_format = "* {language}\n\n{translators}\n"
        translator_format = "    * {1} <{0}> ({2})"
        mime = "text/plain"
        format_html_or_plain = format_plaintext
        format_html_or_plain_join = format_plaintext_join

    language_outputs = []
    for language in data:
        name, translators = language.popitem()
        language_outputs.append(
            format_html_or_plain(
                language_format,
                language=name,
                translators=format_html_or_plain_join(
                    "\n",
                    translator_format,
                    ((t[0], t[1], t[2]) for t in translators),
                ),
            )
        )

    body = format_html_or_plain(
        wrap_format,
        format_html_or_plain_join("\n\n", "{}", ((v,) for v in language_outputs)),
    )
    # Just in case someone messes something up.
    # Also consider simply using `html.unescape` instead.
    if mime != "text/plain":
        body = conditional_escape(body)
    return HttpResponse(body, content_type=f"{mime}; charset=utf-8")


COUNT_DEFAULTS = {
    field: 0
    for field in (
        "t_chars",
        "t_words",
        "chars",
        "words",
        "edits",
        "count",
        "t_chars_new",
        "t_words_new",
        "chars_new",
        "words_new",
        "edits_new",
        "count_new",
        "t_chars_approve",
        "t_words_approve",
        "chars_approve",
        "words_approve",
        "edits_approve",
        "count_approve",
        "t_chars_edit",
        "t_words_edit",
        "chars_edit",
        "words_edit",
        "edits_edit",
        "count_edit",
    )
}


def generate_counts(user, start_date, end_date, **kwargs):
    """Generate credits data for given component."""
    result = {}
    action_map = {Change.ACTION_NEW: "new", Change.ACTION_APPROVE: "approve"}

    base = Change.objects.content().filter(unit__isnull=False)
    if user:
        base = base.filter(author=user)
    else:
        base = base.filter(author__isnull=False)

    changes = base.filter(
        timestamp__range=(start_date, end_date), **kwargs
    ).prefetch_related("author", "unit")
    for change in changes:
        email = change.author.email

        if email not in result:
            result[email] = current = {"name": change.author.full_name, "email": email}
            current.update(COUNT_DEFAULTS)
        else:
            current = result[email]

        src_chars = len(change.unit.source)
        src_words = change.unit.num_words
        tgt_chars = len(change.target)
        tgt_words = len(change.target.split())
        edits = change.get_distance()

        current["chars"] += src_chars
        current["words"] += src_words
        current["t_chars"] += tgt_chars
        current["t_words"] += tgt_words
        current["edits"] += edits
        current["count"] += 1

        suffix = action_map.get(change.action, "edit")

        current["t_chars_" + suffix] += tgt_chars
        current["t_words_" + suffix] += tgt_words
        current["chars_" + suffix] += src_chars
        current["words_" + suffix] += src_words
        current["edits_" + suffix] += edits
        current["count_" + suffix] += 1

    return list(result.values())


@login_required
@require_POST
def get_counts(request, project=None, component=None):
    """View for work counts."""
    if project is None:
        obj = None
        kwargs = {}
    elif component is None:
        obj = get_project(request, project)
        kwargs = {"project": obj}
    else:
        obj = get_component(request, project, component)
        kwargs = {"component": obj}

    form = ReportsForm(request.POST)

    if not form.is_valid():
        show_form_errors(request, form)
        return redirect_param(obj or "home", "#reports")

    data = generate_counts(
        None if request.user.has_perm("reports.view", obj) else request.user,
        form.cleaned_data["start_date"],
        form.cleaned_data["end_date"],
        **kwargs,
    )

    if form.cleaned_data["style"] == "json":
        return JsonResponse(data=data, safe=False)

    headers = (
        "Name",
        "Email",
        "Count total",
        "Edits total",
        "Source words total",
        "Source chars total",
        "Target words total",
        "Target chars total",
        "Count new",
        "Edits new",
        "Source words new",
        "Source chars new",
        "Target words new",
        "Target chars new",
        "Count approved",
        "Edits approved",
        "Source words approved",
        "Source chars approved",
        "Target words approved",
        "Target chars approved",
        "Count edited",
        "Edits edited",
        "Source words edited",
        "Source chars edited",
        "Target words edited",
        "Target chars edited",
    )

    if form.cleaned_data["style"] == "html":
        start = format_html(
            HTML_HEADING,
            format_html_join("", "<th>{}</th>", ((header,) for header in headers)),
        )
        row_start = "<tr>"
        cell_name = cell_count = "<td>{0}</td>\n"
        row_end = "</tr>"
        mime = "text/html"
        end = "</table>"
        format_html_or_plain = format_html
        format_html_or_plain_join = format_html_join
    else:
        start = "{0}\n{1} {2}\n{0}".format(
            RST_HEADING,
            " ".join(f"{h:40}" for h in headers[:2]),
            " ".join(f"{h:24}" for h in headers[2:]),
        )
        row_start = ""
        cell_name = "{0:40} "
        cell_count = "{0:24} "
        row_end = ""
        mime = "text/plain"
        end = RST_HEADING
        format_html_or_plain = format_plaintext
        format_html_or_plain_join = format_plaintext_join

    result = [start]

    for item in data:
        if row_start:
            result.append(row_start)
        result.append(
            format_html_or_plain_join(
                "",
                "{}",
                (
                    (format_html_or_plain(cell_name, item["name"] or "Anonymous"),),
                    (format_html_or_plain(cell_name, item["email"] or ""),),
                    (format_html_or_plain(cell_count, item["count"]),),
                    (format_html_or_plain(cell_count, item["edits"]),),
                    (format_html_or_plain(cell_count, item["words"]),),
                    (format_html_or_plain(cell_count, item["chars"]),),
                    (format_html_or_plain(cell_count, item["t_words"]),),
                    (format_html_or_plain(cell_count, item["t_chars"]),),
                    (format_html_or_plain(cell_count, item["count_new"]),),
                    (format_html_or_plain(cell_count, item["edits_new"]),),
                    (format_html_or_plain(cell_count, item["words_new"]),),
                    (format_html_or_plain(cell_count, item["chars_new"]),),
                    (format_html_or_plain(cell_count, item["t_words_new"]),),
                    (format_html_or_plain(cell_count, item["t_chars_new"]),),
                    (format_html_or_plain(cell_count, item["count_approve"]),),
                    (format_html_or_plain(cell_count, item["edits_approve"]),),
                    (format_html_or_plain(cell_count, item["words_approve"]),),
                    (format_html_or_plain(cell_count, item["chars_approve"]),),
                    (format_html_or_plain(cell_count, item["t_words_approve"]),),
                    (format_html_or_plain(cell_count, item["t_chars_approve"]),),
                    (format_html_or_plain(cell_count, item["count_edit"]),),
                    (format_html_or_plain(cell_count, item["edits_edit"]),),
                    (format_html_or_plain(cell_count, item["words_edit"]),),
                    (format_html_or_plain(cell_count, item["chars_edit"]),),
                    (format_html_or_plain(cell_count, item["t_words_edit"]),),
                    (format_html_or_plain(cell_count, item["t_chars_edit"]),),
                ),
            )
        )
        if row_end:
            result.append(row_end)

    result.append(end)

    return HttpResponse("\n".join(result), content_type=f"{mime}; charset=utf-8")



# Copyright © Michal Čihař <michal@weblate.org>
#
# SPDX-License-Identifier: GPL-3.0-or-later

import tempfile

from django.test.utils import override_settings

from weblate.utils.files import remove_tree


# Lowercase name to be consistent with Django
# pylint: disable=invalid-name
class tempdir_setting(override_settings):  # noqa
    def __init__(self, setting):
        kwargs = {setting: None}
        super().__init__(**kwargs)
        self._tempdir = None
        self._setting = setting

    def enable(self):
        self._tempdir = tempfile.mkdtemp()
        self.options[self._setting] = self._tempdir
        super().enable()

    def disable(self):
        super().disable()
        if self._tempdir is not None:
            remove_tree(self._tempdir)
            self._tempdir = None



# Copyright © Michal Čihař <michal@weblate.org>
#
# SPDX-License-Identifier: GPL-3.0-or-later

from django.test import TestCase

from weblate.auth.models import AutoGroup, Group, User


class AutoGroupTest(TestCase):
    @staticmethod
    def create_user():
        return User.objects.create_user("test1", "noreply1@weblate.org", "pass")

    def test_default(self):
        user = self.create_user()
        self.assertEqual(user.groups.count(), 2)

    def test_none(self):
        AutoGroup.objects.all().delete()
        user = self.create_user()
        self.assertEqual(user.groups.count(), 0)

    def test_matching(self):
        AutoGroup.objects.create(
            match="^.*@weblate.org", group=Group.objects.get(name="Guests")
        )
        user = self.create_user()
        self.assertEqual(user.groups.count(), 3)

    def test_nonmatching(self):
        AutoGroup.objects.create(
            match="^.*@example.net", group=Group.objects.get(name="Guests")
        )
        user = self.create_user()
        self.assertEqual(user.groups.count(), 2)



# Copyright © Michal Čihař <michal@weblate.org>
#
# SPDX-License-Identifier: GPL-3.0-or-later

# Generated by Django 4.0.1 on 2022-01-26 08:22

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("trans", "0146_alter_component_merge_style"),
        ("weblate_auth", "0019_alter_role_name"),
    ]

    operations = [
        migrations.AddField(
            model_name="group",
            name="defining_project",
            field=models.ForeignKey(
                null=True,
                blank=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="defined_groups",
                to="trans.project",
            ),
        ),
        migrations.AlterField(
            model_name="group",
            name="name",
            field=models.CharField(max_length=150, verbose_name="Name"),
        ),
    ]



# Copyright © Michal Čihař <michal@weblate.org>
#
# SPDX-License-Identifier: GPL-3.0-or-later

"""Git based version control system abstraction for Weblate needs."""


from lxml import etree

PARSER = etree.XMLParser(strip_cdata=False, resolve_entities=False)


def parse_xml(text):
    """Parse XML without resolving entities."""
    return etree.fromstring(text, PARSER)



# Copyright © Michal Čihař <michal@weblate.org>
#
# SPDX-License-Identifier: GPL-3.0-or-later

# Generated by Django 4.1 on 2022-09-15 10:59

from django.db import migrations

import weblate.utils.fields


class Migration(migrations.Migration):

    dependencies = [
        ("billing", "0005_auto_20210512_1955"),
    ]

    operations = [
        migrations.AlterField(
            model_name="billing",
            name="payment",
            field=weblate.utils.fields.JSONField(default=dict, editable=False),
        ),
        migrations.AlterField(
            model_name="invoice",
            name="payment",
            field=weblate.utils.fields.JSONField(default=dict, editable=False),
        ),
    ]
